﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.DistServices
{
    using System.Collections.Generic;
    using TFC.PublicLighting.Services;
    using TFC.PublicLighting.Contracts.DTOs;

    public class PLManagementService : IPLManagementService
    {
        TecnicsManagementService tecnicsService = new TecnicsManagementService();
        SupervisorManagementService supervisorService = new SupervisorManagementService();
        SectorsManagementService sectorsService = new SectorsManagementService();
        ElementsManagementService elementsService = new ElementsManagementService();
        IncidenciesManagementService incidenciesService = new IncidenciesManagementService();
        OrdresManagementService ordresService = new OrdresManagementService();
        LogginService logginService = new LogginService();

        // Llistes per a selecionar a les vistes.
        # region Llistes Vistes

        public List<SectorDTO> GetAllSectors()
        {
            return sectorsService.GetAllSectors();
        }

        public List<TipusElementDTO> GetAllTipusElement()
        {
            return elementsService.GetAllTipusElement();
        }

        public List<TipusAvariaDTO> GetAllTipusAvaria()
        {
            return incidenciesService.GetAllTipusAvaria();
        }

        public List<EstatIncidenciaDTO> GetAllEstatsIncidencia()
        {
            return incidenciesService.GetAllEstatsIncidencia();
        }

        public List<TipusPrioritatDTO> GetAllTipusPrioritat()
        {
            return ordresService.GetAllTipusPrioritat();
        }

        public List<EstatOrdreDTO> GetAllEstatsOrdre()
        {
            return ordresService.GetAllEstatsOrdre();
        }

        # endregion
                
        // Gestió d'Usuaris
        # region Usuaris

        public UsuariDTO GetLoggedUser(string user, string password)
        {
            return logginService.GetLoggedUser(user, password);
        }
        
        public UsuariDTO GetTecnicById(int pId)
        {
            return tecnicsService.GetTecnicById(pId);
        }

        public List<UsuariDTO> GetTecnicsByFilter(UsuariDTO tecnicFilter)
        {
            return tecnicsService.GetTecnicsByFilter(tecnicFilter);
        }

        public UsuariDTO GetSupervisor()
        {
            return supervisorService.GetSupervisor();
        }

        public List<UsuariDTO> GetAllTecnics()
        {
            return tecnicsService.GetAllTecnics();
        }

        public bool AddTecnic(UsuariDTO pTecnic, int usuariDades)
        {
            return tecnicsService.AddTecnic(pTecnic, usuariDades);
        }

        public bool AddSupervisor(UsuariDTO pSupervisor, int usuariDades)
        {
            return supervisorService.AddSupervisor(pSupervisor, usuariDades);
        }

        public bool UpdateSupervisor(UsuariDTO pSupervisor, int usuariDades)
        {
            return supervisorService.UpdateSupervisor(pSupervisor, usuariDades);
        }

        public bool UpdateTecnic(UsuariDTO pTecnic, int usuariDades)
        {
            return tecnicsService.UpdateTecnic(pTecnic, usuariDades);
        }

        public bool DeleteTecnic(int pId, int usuariDades)
        {
            return tecnicsService.DeleteTecnic(pId, usuariDades);
        }

        public bool DeleteSupervisor(int pId, int usuariDades)
        {
            return supervisorService.DeleteSupervisor(pId, usuariDades);
        }

        # endregion

        // Gestió d'Elements
        # region Elements

        public ElementDTO GetElementById(int pId)
        {
            return elementsService.GetElementById(pId);
        }

        public List<ElementDTO> GetAllElements()
        {
            return elementsService.GetAllElements();
        }

        public List<ElementDTO> GetElementsByFilter(ElementDTO elementFilter)
        {
            return elementsService.GetElementsByFilter(elementFilter);
        }

        public bool AddElement(ElementDTO pElement, int usuariDades)
        {
            return elementsService.AddElement(pElement, usuariDades);
        }

        public bool UpdateElement(ElementDTO pElement, int usuariDades)
        {
            return elementsService.UpdateElement(pElement, usuariDades);
        }

        public bool DeleteElement(int pId, int usuariDades)
        {
            return elementsService.DeleteElement(pId, usuariDades);
        }

        # endregion

        // Gestió d'Incidències
        # region Incidencies

        public IncidenciaDTO GetIncidenciaById(int pId)
        {
            return incidenciesService.GetIncidencia(pId);
        }

        public List<IncidenciaDTO> GetAllIncidencies()
        {
            return incidenciesService.GetAllIncidencies();
        }

        public List<IncidenciaDTO> GetIncidenciesByFilter(IncidenciaDTO incidenciaFilter)
        {
            return incidenciesService.GetIncidenciesByFilter(incidenciaFilter);
        }

        public bool AddIncidencia(IncidenciaDTO incidenciaDTO, int usuariDades)
        {
            return incidenciesService.AddIncidencia(incidenciaDTO, usuariDades);
        }

        public bool UpdateIncidencia(IncidenciaDTO incidenciaDTO, int usuariDades)
        {
            return incidenciesService.UpdateIncidencia(incidenciaDTO, usuariDades);
        }

        public bool DeleteIncidencia(int pId, int usuariDades)
        {
            return incidenciesService.DeleteIncidencia(pId, usuariDades);
        }

        # endregion

        // Gestió d'Ordres
        # region Ordres

        public OrdreDTO GetOrdreById(int pId)
        {
            return ordresService.GetOrdre(pId);
        }

        public List<OrdreDTO> GetAllOrdres()
        {
            return ordresService.GetAllOrdres();
        }

        public List<OrdreDTO> GetOrdresByFilter(OrdreDTO ordreFilter)
        {
            return ordresService.GetOrdresByFilter(ordreFilter);
        }

        public List<OrdreDTO> GetAllOrdresByTecnicId( int pIdTecnic)
        {
            return ordresService.GetAllOrdresByTecnicId(pIdTecnic);
        }

        public bool AddOrdre(OrdreDTO ordreDTO, int usuariDades)
        {
            return ordresService.AddOrdre(ordreDTO, usuariDades);
        }

        public bool UpdateOrdre(OrdreDTO ordreDTO, int usuariDades)
        {
            return ordresService.UpdateOrdre(ordreDTO, usuariDades);
        }

        public bool UpdateListOrdres(List<OrdreDTO> pOrdreList, int usuariDades)
        {
            return ordresService.UpdateListOrdres(pOrdreList, usuariDades);
        }

        public bool DeleteOrdre(int pId, int usuariDades)
        {
            return ordresService.DeleteOrdre(pId, usuariDades);
        }

        # endregion



    }
}
