﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.Infraestructure
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using TFC.PublicLighting.Domain;

    public class ElementsRepository
    {
        #region Model Attributes
        private static PublicLightingEntities context;
        # endregion

        public List<tipusElement> GetAllTipusElement()
        {
            using (context = new PublicLightingEntities())
            {
                List<tipusElement> result = (from e in context.tipusElement
                                             orderby e.ordreVisualitzacio
                                             select e).ToList();

                return result;
            }
        }

        public List<elementEnllumenat> GetAllElements()
        {
            using (context = new PublicLightingEntities())
            {
                List<elementEnllumenat> elements = (from e in context.elementEnllumenat
                                                    where e.estatDades != Constants.DataState.Removed
                                                    select e).ToList();

                return elements;
            }
        }

        public elementEnllumenat GetElement(int pId)
        {
            using (context = new PublicLightingEntities())
            {
                elementEnllumenat element = (from e in context.elementEnllumenat
                                              where e.id_element == pId
                                              && e.estatDades != Constants.DataState.Removed
                                              select e).SingleOrDefault();

                return element;
            }
        }

        public bool AddElement(elementEnllumenat element, int idUsuari)
        {
            bool completat = true;
            using (context = new PublicLightingEntities())
            {
                // Obté el sector de la xarxa o es llença una excepcio.
                sectorXarxa sector = (from s in context.sectorXarxa
                                       where s.id_sectorXarxa == element.id_sectorXarxa
                                       select s).Single();

                // Obté el tipus d'element o es llença una excepcio.
                tipusElement tipus = (from t in context.tipusElement
                                       where t.id_tipusElement == element.id_tipusElement
                                       select t).Single();

                // Obté l'usuari modificador de les dades o es llença una excepcio.
                usuari usuariDades = (from u in context.usuari
                                      where u.id_usuari == idUsuari
                                      && u.estatDades != Constants.DataState.Removed
                                      select u).Single();

                element.sectorXarxa = sector;
                element.tipusElement = tipus;

                element.usuari = usuariDades;
                element.dataDades = DateTime.Now;
                element.estatDades = Constants.DataState.Created;

                context.elementEnllumenat.AddObject(element);

                try
                {
                    context.SaveChanges();
                }
                catch (Exception)
                {
                    completat = false;
                }
            }

            return completat;
        }

        public bool UpdateElement(elementEnllumenat element, int idUsuari)
        {
            bool completat = true;
            using (context = new PublicLightingEntities())
            {
                // Obté l'element coincident o es llença una excepcio.
                elementEnllumenat currentElement = (from e in context.elementEnllumenat
                                                     where e.id_element == element.id_element
                                                      && e.estatDades != Constants.DataState.Removed
                                                      select e).Single();
                // Obté el sector de la xarxa o es llença una excepcio.
                sectorXarxa sector = (from s in context.sectorXarxa
                                   where s.id_sectorXarxa == element.id_sectorXarxa
                                   select s).Single();

                // Obté el tipus d'element o es llença una excepcio.
                tipusElement tipus = (from t in context.tipusElement
                                   where t.id_tipusElement == element.id_tipusElement
                                   select t).Single();

                // Obté l'usuari modificador de les dades o es llença una excepcio.
                usuari usuariDades = (from u in context.usuari
                                     where u.id_usuari == idUsuari
                                     && u.estatDades != Constants.DataState.Removed
                                     select u).Single();

                currentElement.sectorXarxa = sector;
                currentElement.tipusElement = tipus;
                currentElement.carrer = element.carrer;
                currentElement.codiPostal = element.codiPostal;
                currentElement.fabricant = element.fabricant;
                currentElement.matricula = element.matricula;
                currentElement.notesPosicio = element.notesPosicio;
                currentElement.numeroAproximat = element.numeroAproximat;
                currentElement.posicioLatitud = element.posicioLatitud;
                currentElement.posicioLongitud = element.posicioLongitud;
                currentElement.referencia = element.referencia;

                currentElement.usuari = usuariDades;
                currentElement.dataDades = System.DateTime.Now;
                currentElement.estatDades = Constants.DataState.Modified;

                try
                {
                    context.SaveChanges();
                }
                catch (Exception)
                {
                    completat = false;
                }
            }
            return completat;
        }

        public bool DeleteElement(int id_element, int idUsuari)
        {
            bool succes = false;
            using (context = new PublicLightingEntities())
            {
                // Obté l'element per esborrar o es llença una excepcio.
                elementEnllumenat elementToDelete = (from e in context.elementEnllumenat
                                                   where e.id_element == id_element
                                                   && e.estatDades != Constants.DataState.Removed
                                                   select e).Single();
               
                // Obté l'usuari modificador de les dades o es llença una excepcio.
                usuari usuariDades = (from u in context.usuari
                                      where u.id_usuari == idUsuari
                                      && u.estatDades != Constants.DataState.Removed
                                      select u).Single();


                elementToDelete.estatDades = Constants.DataState.Removed;
                elementToDelete.dataDades = DateTime.Now;
                elementToDelete.usuari = usuariDades;

                try
                {
                    context.SaveChanges();
                }
                catch (Exception)
                {
                    succes = false;
                }
            }
            return succes;
        }
    }
}
