﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.Infraestructure
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using TFC.PublicLighting.Domain;

    public class IncidenciesRepository
    {
        #region Model Attributes
        private static PublicLightingEntities context;
        # endregion

        public List<tipusAvaria> GetAllTipusAvaria()
        {
            using (context = new PublicLightingEntities())
            {
                List<tipusAvaria> allTipusAvaria = (from t in context.tipusAvaria
                                                    orderby t.ordre_visualitzacio
                                                    select t).ToList();
                return allTipusAvaria;
            }
        }

        public List<estatIncidencia> GetAllEstatsIncidencia()
        {
            using (context = new PublicLightingEntities())
            {
                List<estatIncidencia> result = (from e in context.estatIncidencia
                                                orderby e.ordreVisualitzacio
                                               select e).ToList();
                return result;
            }
        }

        public incidencia GetIncidencia(int pId)
        {
            using (context = new PublicLightingEntities())
            {
                incidencia incidencia = (from i in context.incidencia
                                          where i.id_incidencia == pId
                                          && i.estatDades != Constants.DataState.Removed
                                          select i).SingleOrDefault();
                return incidencia;
            }
        }

        public List<incidencia> GetAllIncidencies()
        {
            using (context = new PublicLightingEntities())
            {
                List<incidencia> allIncidenciasList = (from i in context.incidencia
                                                       where i.estatDades != Constants.DataState.Removed
                                                       select i).ToList();

                return allIncidenciasList;
            }
        }

        public bool AddIncidencia(incidencia incidencia, int idUsuari)
        {
            bool succes = true;
            using (context = new PublicLightingEntities())
            {
                // Comprova que l'element existeix o es llença una excepcio.
                elementEnllumenat element = (from e in context.elementEnllumenat
                                           where e.id_element == incidencia.id_element
                                           && e.estatDades != Constants.DataState.Removed
                                           select e).Single();               

                // Comprova que l'usuari es el Supervisor o es llença una excepcio.
                usuari supervisor = (from u in context.usuari
                                  where u.id_usuari == idUsuari
                                  && u.estatDades != Constants.DataState.Removed
                                  select u).Single();

                // Comprova que l'avaria existeix o es llença una excepcio.
                tipusAvaria tipusAvaria = (from t in context.tipusAvaria
                                   where t.id_tipus_avaria == incidencia.id_tipusAvaria
                                   select t).Single();

                incidencia.elementEnllumenat = element;
                incidencia.usuariDades = supervisor;
                incidencia.usuariSupervisor = supervisor;
                incidencia.dataDades = System.DateTime.Now;
                incidencia.estatDades = Constants.DataState.Created;

                context.incidencia.AddObject(incidencia);

                try
                {
                    context.SaveChanges();
                }
                catch (Exception)
                {
                    succes = false;
                }
            }
            return succes;
        }

        public bool UpdateIncidencia(incidencia incidencia, int idUsuari)
        {
            bool succes = true;
            using (context = new PublicLightingEntities())
            {
                // Comprova que la incidència existeix o es llença una excepció.
                incidencia updatedIncidencia = (from i in context.incidencia
                                                where i.id_incidencia == incidencia.id_incidencia
                                                && i.estatDades != Constants.DataState.Removed
                                                select i).Single();

                // Comprova que l'element existeix o es llença una excepció.
                elementEnllumenat element = (from e in context.elementEnllumenat
                                               where e.id_element == incidencia.id_element
                                               && e.estatDades != Constants.DataState.Removed
                                               select e).Single();

                // Obté l'usuari Supervisor o es llença una excepció.
                usuari supervisor = (from u in context.usuari
                                    where u.id_usuari == idUsuari
                                    && u.estatDades != Constants.DataState.Removed
                                    select u).Single();

                // Obté l'avaria o es llença una excepció.
                tipusAvaria tipus = (from t in context.tipusAvaria
                                       where t.id_tipus_avaria == incidencia.id_tipusAvaria
                                       select t).Single();

                // Actualitza la incidència.
                updatedIncidencia.elementEnllumenat = element;
                updatedIncidencia.usuariDades = supervisor;
                updatedIncidencia.usuariSupervisor = supervisor;
                updatedIncidencia.tipusAvaria = tipus;
                updatedIncidencia.comentari = incidencia.comentari;
                updatedIncidencia.ordreReparacio = incidencia.ordreReparacio;
                updatedIncidencia.dataCreacio = incidencia.dataCreacio;

                updatedIncidencia.usuariDades = supervisor;
                updatedIncidencia.dataDades = System.DateTime.Now;
                updatedIncidencia.estatDades = Constants.DataState.Modified;

                try
                {
                    context.SaveChanges();
                }
                catch (Exception)
                {
                    succes = false;
                }
            }
            return succes;
        }

        public bool DeleteIncidencia(int id, int idUsuari)
        {
            bool succes = true;
            using (context = new PublicLightingEntities())
            {
                // Comprova que la incidència existeix o es llença una excepcio.
                incidencia deletedIncidencia = (from i in context.incidencia
                                                where i.id_incidencia == id
                                                && i.estatDades != Constants.DataState.Removed
                                                select i).Single();

                // Comprova que l'usuari es el Supervisor
                usuari supervisor = (from u in context.usuari
                                     where u.id_usuari == idUsuari
                                     && u.estatDades != Constants.DataState.Removed
                                     select u).Single();

                // Es fa l'esborrat lògic.
                deletedIncidencia.estatDades = Constants.DataState.Removed;
                deletedIncidencia.dataDades = DateTime.Now;
                deletedIncidencia.usuariDades = supervisor;

                try
                {
                    context.SaveChanges();
                }
                catch (Exception)
                {
                    succes = false;
                }
            }
            return succes;
        }
    }
}
