﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.Infraestructure
{
    using System.Collections.Generic;
    using System.Linq;
    using System;
    using TFC.PublicLighting.Domain;

    public class OrdresRepository
    {
        #region Model Attributes
        private static PublicLightingEntities context;
        # endregion

        public List<tipusPrioritat> GetAllTipusPrioritat()
        {
            using (context = new PublicLightingEntities())
            {
                List<tipusPrioritat> allPrioritats = (from p in context.tipusPrioritat
                                                       orderby p.ordre_visualitzacio
                                                       select p).ToList();
                return allPrioritats;
            }
        }
                
        public List<estatOrdre> GetAllEstatsOrdre()
        {
            using (context = new PublicLightingEntities())
            {
                List<estatOrdre> allEstatsList = (from e in context.estatOrdre
                                                    orderby e.ordre_visualitzacio
                                                    select e).ToList();
                return allEstatsList;
            }
        }

        public ordreReparacio GetOrdreById(int id)
        {
            using (context = new PublicLightingEntities())
            {
                ordreReparacio ordre = (from o in context.ordreReparacio
                                        .Include("incidencies")
                                        where o.id_ordreReparacio == id
                                        && o.estatDades != Constants.DataState.Removed
                                        select o).Single();
                return ordre;
            }
        }

        public List<ordreReparacio> GetAllOrdres()
        {
            using (context = new PublicLightingEntities())
            {
                List<ordreReparacio> allOrdresList = (from o in context.ordreReparacio
                                                    .Include("incidencies")
                                                    where o.estatDades != Constants.DataState.Removed
                                                    select o).ToList();
                return allOrdresList;
            }
        }

        public List<ordreReparacio> GetAllOrdresByTecnicId(int id)
        {
            using (context = new PublicLightingEntities())
            {
                List<ordreReparacio> ordres = (from o in context.ordreReparacio
                                                .Include("incidencies")
                                                where o.estatDades != Constants.DataState.Removed
                                                && o.id_tecnic == id
                                                select o).ToList();
                return ordres;
            }
        }

        public bool AddOrdre(ordreReparacio newOrdre, List<int> IdIncidenciesList, int idUsuari)
        {
            bool succes = true;
            using (context = new PublicLightingEntities())
            {
                // Comprova que l'estat existeix o es llença una excepcio. 
                estatOrdre estat = (from e in context.estatOrdre
                                      where e.id_estat_ordre == newOrdre.id_estatOrdre
                                      select e).Single();

                // Comprova que el tipus de prioritat existeix o es llença una excepcio. 
                tipusPrioritat tipus = (from t in context.tipusPrioritat
                                      where t.id_tipusPrioritat == newOrdre.id_tipusPrioritat
                                      select t).Single();

                // Obté l'usuari Supervisor i modificador de les dades o es llença una excepcio.
                usuari supervisor = (from u in context.usuari
                                     where u.id_usuari == idUsuari
                                     && u.estatDades != Constants.DataState.Removed
                                     select u).Single();

                // Obté l'usuari Tecnic o es llença una excepcio.
                usuari tecnic = (from u in context.usuari
                                 where u.id_usuari == newOrdre.id_tecnic
                                 && u.estatDades != Constants.DataState.Removed
                                 select u).Single();

                newOrdre.estatOrdre = estat;
                newOrdre.tipusPrioritat = tipus;
                newOrdre.usuariSupervisor = supervisor;
                newOrdre.usuariTecnic = tecnic;

                newOrdre.estatDades = Constants.DataState.Created;
                newOrdre.dataDades = DateTime.Now;
                newOrdre.usuariDades = supervisor;

                // Afegeix totes les incidències relacionades.
                foreach (int id in IdIncidenciesList)
                {
                    incidencia incidencia = (from inc in context.incidencia
                                             where inc.id_incidencia == id
                                             && inc.estatDades != Constants.DataState.Removed
                                             select inc).Single();

                    newOrdre.incidencies.Add(incidencia);
                }

                context.ordreReparacio.AddObject(newOrdre);

                try
                {
                    context.SaveChanges();
                }
                catch (Exception)
                {
                    succes = false;
                }
            }
            return succes;
        }


        public bool UpdateOrdre(ordreReparacio ordre, List<int> IdIncidenciesList, int idUsuari)
        {
            bool succes = true;
            using (context = new PublicLightingEntities())
            {
                // Obté l'ordre coincident
                ordreReparacio currentOrdre = (from o in context.ordreReparacio
                                                where o.id_ordreReparacio == ordre.id_ordreReparacio
                                                && o.estatDades != Constants.DataState.Removed
                                                select o).Single();
                // Obté el'estat
                estatOrdre estat = (from e in context.estatOrdre
                                    where e.id_estat_ordre == ordre.id_estatOrdre
                                    select e).Single();

                // Obté la prioritat
                tipusPrioritat prioritat = (from t in context.tipusPrioritat
                                            where t.id_tipusPrioritat == ordre.id_tipusPrioritat
                                            select t).Single();

                // Obté l'usuari Supervisor o es llença una excepcio.
                usuari supervisor = (from u in context.usuari
                                     where u.id_usuari == idUsuari
                                     && u.estatDades != Constants.DataState.Removed
                                     select u).Single();

                // Obté l'usuari Tecnic o es llença una excepcio.
                usuari tecnic = (from u in context.usuari
                                     where u.id_usuari == ordre.id_tecnic
                                     && u.estatDades != Constants.DataState.Removed
                                     select u).Single();

                // Esborra la llista actual d'incidències
                currentOrdre.incidencies.Clear();

                // Actualitza l'ordre.

                // Afegeix totes les incidències relacionades.
                foreach (int id in IdIncidenciesList)
                {
                    incidencia incidencia = (from inc in context.incidencia
                                             where inc.id_incidencia == id
                                             && inc.estatDades != Constants.DataState.Removed
                                             select inc).Single();

                    currentOrdre.incidencies.Add(incidencia);
                }
                
                currentOrdre.comentariSupervisor = ordre.comentariSupervisor;
                currentOrdre.comentariTecnic = ordre.comentariTecnic;
                currentOrdre.dataCreacio = ordre.dataCreacio;
                currentOrdre.dataResolucio = ordre.dataResolucio;
                currentOrdre.usuariSupervisor = supervisor;                
                currentOrdre.usuariTecnic = tecnic;
                currentOrdre.estatOrdre = estat;
                currentOrdre.tipusPrioritat = prioritat;

                currentOrdre.estatDades = Constants.DataState.Modified;
                currentOrdre.dataDades = DateTime.Now;
                currentOrdre.usuariDades = supervisor;

                try
                {
                    context.SaveChanges();
                }
                catch (Exception)
                {
                    succes = false;
                }
            }
            return succes;
        }

        public bool DeleteOrdre(int idOrdre, int idUsuariDades)
        {
            bool succes = false;
            using (context = new PublicLightingEntities())
            {
                // PAS 1. Obté l'ordre coincident i si no existeix llença una excepció.
                ordreReparacio currentOrdre = (from o in context.ordreReparacio
                                               where o.id_ordreReparacio == idOrdre
                                               && o.estatDades != Constants.DataState.Removed
                                               select o).Single();

                // PAS 2. Obté l'usuari modificador de les dades o es llença una excepcio.
                usuari usuariDades = (from u in context.usuari
                                      where u.id_usuari == idUsuariDades
                                      && u.estatDades != Constants.DataState.Removed
                                      select u).Single();
                
                if (currentOrdre != null)
                {
                    // PAS 3. Canvi a estat Eliminat.
                    currentOrdre.estatDades = Constants.DataState.Removed;
                    currentOrdre.dataDades = DateTime.Now;
                    currentOrdre.usuariDades = usuariDades;

                    // PAS 4. Esborra la llista actual d'incidències
                    currentOrdre.incidencies.Clear();

                    // PAS 5. Enregistra els canvis a la base de dades.
                    try
                    {
                        context.SaveChanges();
                        succes = true;
                    }
                    catch (Exception)
                    {
                        succes = false;
                    }
                }
                return succes;
            }
        }
    }
}
