﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.Infraestructure
{
using System;
using System.Collections.Generic;
using System.Linq;
using TFC.PublicLighting.Domain;

    public class UsuarisRepository
    {
        #region Model Attributes
        private static PublicLightingEntities context; 
        # endregion

        public usuari GetLoggedUser(string user, string password)
        {
            using (context = new PublicLightingEntities())
            {
                // PAS 1. Obtenció de l'usuari existent.
                usuari loggedUser = (from e in context.usuari
                                      where e.nomUsuari == user
                                      && e.clau == password
                                      && e.estatDades != Constants.DataState.Removed
                                      select e).SingleOrDefault();
                return loggedUser;
            }
        }

        public usuari GetSupervisor()
        {
            using (context = new PublicLightingEntities())
            {
                // PAS 1. Obtenció de l'usuari existent.
                usuari supervisor = (from e in context.usuari
                                      where e.id_tipusUsuari == Constants.UserType.Super
                                      && e.estatDades != Constants.DataState.Removed
                                      select e).SingleOrDefault();
                return supervisor;
            }
        }

        public usuari GetUserByTypeAndId(int id, string tipus)
        {
            using (context = new PublicLightingEntities())
            {
                usuari usuari = (from e in context.usuari
                                  where e.id_usuari == id
                                  && e.id_tipusUsuari == tipus
                                  && e.estatDades != Constants.DataState.Removed
                                  select e).Single();
                return usuari;
            }
        }

        public usuari GetTecnicById(int pId)
        {
            using (context = new PublicLightingEntities())
            {
                // PAS 1. Obtenció de l'usuari existent.
                usuari tecnic = (from e in context.usuari
                                  where e.id_usuari == pId
                                  && e.id_tipusUsuari == Constants.UserType.Tec
                                  && e.estatDades != Constants.DataState.Removed
                                  select e).SingleOrDefault();

                return tecnic;
            }
        }               

        /// <summary>
        /// Retorna tots els usuaris del tipus indicat.
        /// </summary>
        /// <returns>
        /// Llista d'usuaris.
        /// </returns>
        public List<usuari> GetAllUsuaris(string tipus)
        {
            using (context = new PublicLightingEntities())
            {
                List<usuari> result = (from e in context.usuari
                                        where e.id_tipusUsuari == tipus
                                        && e.estatDades != Constants.DataState.Removed
                                        select e).ToList();

                return result;
            }
        }

        /// <summary>
        /// Afegeix un Usuari.
        /// </summary>
        /// <returns>
        /// True si s'ha succes l'operació, false en cas contrari.
        /// <returns>
        public bool AddUsuari(usuari usuari, int usuariDades)
        {
            bool succes = true;
            using (context = new PublicLightingEntities())
            {
                // PAS 1. Comprovació de conflictes amb usuaris existents.
                var selUsuari = (from e in context.usuari
                              where e.documentId == usuari.documentId
                              || (e.id_usuari == usuari.id_usuari && usuari.id_usuari != 0)
                              select e).SingleOrDefault();

                if (selUsuari != null)
                {
                    succes = false;
                }
                else
                {
                    // PAS 2. Actualització de les dades d'enregistrament.
                    usuari.estatDades = Constants.DataState.Modified;
                    usuari.dataDades = DateTime.Now;
                    usuari.id_usuariDades = usuariDades;
                    
                     //PAS 3. Inserció del nou usuari.
                    context.usuari.AddObject(usuari);

                    // PAS 4. Enregistra els canvis a la base de dades.
                    try
                    {
                        context.SaveChanges();
                    }
                    catch (Exception ex)
                    {
                        succes = false;
                    }                                                
                 }                                                
            }
            return succes;
        }

        /// <summary>
        /// Actualitza l'Usuari identificat pel seu Id.
        /// </summary>
        /// <param name="Id">L'Id que identifica l'Usuari.</param>
        /// <returns>
        /// True si s'ha succes l'operació, False en cas contrari.
        /// </returns>
        public bool UpdateUsuari(usuari usuari, int usuariDades)
        {
            bool succes = false;
            using (context = new PublicLightingEntities())
            {
                // PAS 1. Obtenció de l'objecte usuari existent.
                var updatedUsuari = (from e in context.usuari
                                  where e.id_usuari == usuari.id_usuari
                                  && e.id_tipusUsuari == usuari.id_tipusUsuari
                                  select e).SingleOrDefault(); 

                if (updatedUsuari != null)
                {
                    // PAS 2. Actualització de les dades.
                    updatedUsuari.nom = usuari.nom;
                    updatedUsuari.cognom1 = usuari.cognom1;
                    updatedUsuari.cognom2 = usuari.cognom2;
                    updatedUsuari.id_tipusDocument = usuari.id_tipusDocument;
                    updatedUsuari.documentId = usuari.documentId;
                    updatedUsuari.dataNeixement = usuari.dataNeixement;
                    updatedUsuari.categoria = usuari.categoria;
                    updatedUsuari.adreça = usuari.adreça;
                    updatedUsuari.poblacio = usuari.poblacio;
                    updatedUsuari.codiPostal = usuari.codiPostal;
                    updatedUsuari.provincia = usuari.provincia;
                    updatedUsuari.telefonPrivat = usuari.telefonPrivat;
                    updatedUsuari.mobilEmpresa = usuari.mobilEmpresa;
                    updatedUsuari.email = usuari.email;
                    updatedUsuari.nomUsuari = usuari.nomUsuari;
                    updatedUsuari.id_tipusUsuari = usuari.id_tipusUsuari;
                    updatedUsuari.clau = usuari.clau;
                    updatedUsuari.id_sector1 = usuari.id_sector1;
                    updatedUsuari.id_sector2 = usuari.id_sector2;
                    updatedUsuari.id_sector3 = usuari.id_sector3;

                    // PAS 3. Actualització de les dades d'enregistrament.
                    updatedUsuari.estatDades = Constants.DataState.Modified;
                    updatedUsuari.dataDades = DateTime.Now;
                    updatedUsuari.id_usuariDades = usuariDades;

                    // PAS 4. Enregistra els canvis a la base de dades.
                    try
                    {
                        context.SaveChanges();
                        succes = true;
                    }
                    catch (Exception ex)
                    {
                        succes = false;
                    }
                }
                return succes;
            }           
        }

        public bool DeleteUsuari(int id, int idUsuariDades)
        {
            bool succes = false;
            using (context = new PublicLightingEntities())
            {
                // PAS 1. Obtenció de l'objecte usuari existent.
                usuari deletedUsuari = (from e in context.usuari
                                      where e.id_usuari == id
                                      && e.estatDades != Constants.DataState.Removed
                                      select e).SingleOrDefault();

                // PAS 2. Obté l'usuari modificador de les dades o es llença una excepcio.
                usuari usuariDades = (from u in context.usuari
                                      where u.id_usuari == idUsuariDades
                                      && u.estatDades != Constants.DataState.Removed
                                      select u).Single();
                
                if (deletedUsuari != null)
                {
                    // PAS 3. Canvi a estat Eliminat.
                    deletedUsuari.estatDades = Constants.DataState.Removed;
                    deletedUsuari.dataDades = DateTime.Now;
                    deletedUsuari.usuariDades = usuariDades;

                    // PAS 4. Enregistra els canvis a la base de dades.
                    try
                    {
                        context.SaveChanges();
                        succes = true;
                    }
                    catch (Exception)
                    {
                        succes = false;
                    }
                }
                return succes;
            }
        }

        public void SetUsuarisAsRemovedByType(string tipus, int idUsuariDades)
        {
            using (context = new PublicLightingEntities())
            {
                List<usuari> usersToBeRemoved = (from u in context.usuari
                                                where u.id_tipusUsuari == tipus
                                                select u).ToList();

                foreach(usuari u in usersToBeRemoved)
                {
                    u.estatDades = Constants.DataState.Removed;
                    u.dataDades = DateTime.Now;
                    u.id_usuariDades = idUsuariDades;
                }

                context.SaveChanges();
            }
        }
    }
} 
