﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.PresentationWPF.Element
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.ComponentModel.Composition;
    using System.Windows;
    using Caliburn.Micro;
    using TFC.PublicLighting.PresentationWPF.Cache;
    using TFC.PublicLighting.PresentationWPF.Events;
    using TFC.PublicLighting.PresentationWPF.PLManagementServiceReference;

    [Export(typeof(ElementViewModel))]
    public class ElementViewModel : Screen, IDataErrorInfo, IHandle<CommandEvent>
    {
        private readonly IWindowManager windowManager;
        private IEventAggregator events;

        private string _formErrors;
        private string _lastError = "Formulari incomplert o amb errors";
        private List<SectorDTO> _sectorsList;
        private List<TipusElementDTO> _tipusList;
        private string _controlsAdministrador;

        // SERVEI CLIENT per interactuar amb els serveis distribuits.
        PLManagementServiceClient client = new PLManagementServiceClient();

        // MODEL de dades del ViewModel
        private ElementDTO _elementModel;

        [ImportingConstructor]
        public ElementViewModel(IEventAggregator pEvents, IWindowManager pWindowManager)
        {
            // Obté l'agregador d'esdeveniments a través del constructor i subscriu aquest ViewModel
            // perquè pugui escoltar els missatges d'esdeveniments.
            events = pEvents;
            events.Subscribe(this);
            
            windowManager = pWindowManager;

            // Comproba quin tipus d'usuari és per activar els controls corresponents.
            if (CacheMain.IsAdministradorOn)
            {
                ControlsAdministrador = "Visible";
            }
            else
            {
                ControlsAdministrador = "Collapsed";
            }
            
            SectorsList = client.GetAllSectors();
            SectorDTO noSector = new SectorDTO() { IdSector = 0, NomBarri = "-" };
            SectorsList.Insert(0, noSector);

            TipusList = client.GetAllTipusElement();
        }

        # region VIEW

        // Accions a efectuar quan la vista ha estat enllaçada al ViewModel
        protected override void OnViewAttached(object view, object context)
        {
            base.OnViewAttached(view, context);

            // Comprova si s'ha de fer servir el caché del model.
            if (CacheMain.IsNewElement)
            {
                ElementModel = new ElementDTO() { Id = 0, Id_tipusElement = 1, Id_sectorXarxa = 1 };
                CacheMain.IsNewElement = false;
                CacheMain.CurrentElement = ElementModel;
            }
            else
            {
                ElementModel = CacheMain.CurrentElement;
            }
        }

        // Accions a efectuar quan es surt de la vista
        protected override void OnDeactivate(bool close)
        {
            CacheMain.CurrentElement = ElementModel;
            base.OnDeactivate(close);
        }

        # endregion

         # region Error
         public string FormErrors
         {
             get { return _formErrors; }
             set
             {
                 _formErrors = value;
                 NotifyOfPropertyChange(() => FormErrors);
             }
         }

         public string LastError
         {
             get { return _lastError; }
             set
             {
                 _lastError = value;
                 NotifyOfPropertyChange(() => LastError);
             }
         }
         # endregion

         # region Properties

         public ElementDTO ElementModel
         {
             get { return _elementModel; }
             set
             {
                 _elementModel = value;
                 NotifyOfPropertyChange(() => ElementModel);
             }
         }

         public string ControlsAdministrador
         {
             get { return _controlsAdministrador; }
             set
             {
                 _controlsAdministrador = value;
                 NotifyOfPropertyChange(() => ControlsAdministrador);
             }
         }

         public List<SectorDTO> SectorsList
         {
             get
             {
                 return _sectorsList;
             }
             set
             {
                 _sectorsList = value;
                 NotifyOfPropertyChange(() => SectorsList);
             }
         }

         public List<TipusElementDTO> TipusList
         {
             get
             {
                 return _tipusList;
             }
             set
             {
                 _tipusList = value;
                 NotifyOfPropertyChange(() => TipusList);
             }
         }

         /// <summary>
         /// Identificació de l'Element a la BBDD
         /// </summary>
         public int Id
         {
             get { return ElementModel.Id; }
             set
             {
                 ElementModel.Id = value;
                 NotifyOfPropertyChange(() => Id);
                 NotifyOfPropertyChange(() => CanSave);

                 if (value < 0)
                 {
                     LastError = "L'Id és incorrecte";
                     _dataErrors["Id"] = LastError;
                 }
                 else
                 {
                     if (_dataErrors.ContainsKey("Id"))
                         _dataErrors.Remove("Id");
                 }
             }
         }

         public int Tipus
         {
             get { return ElementModel.Id_tipusElement; }
             set
             {
                 ElementModel.Id_tipusElement = value;
                 NotifyOfPropertyChange(() => Tipus);
                 NotifyOfPropertyChange(() => CanSave);

                 if (value == 0)
                 {
                     LastError = "Es requereix el Tipus";
                     _dataErrors["Tipus"] = LastError;
                 }
                 else
                 {
                     if (_dataErrors.ContainsKey("Tipus"))
                         _dataErrors.Remove("Tipus");
                 }
             }
         }

         public string Matricula
         {
             get { return ElementModel.Matricula; }
             set
             {
                 ElementModel.Matricula = value;
                 NotifyOfPropertyChange(() => Matricula);
             }
         }

         public string Fabricant
         {
             get { return ElementModel.Fabricant; }
             set
             {
                 ElementModel.Fabricant = value;
                 NotifyOfPropertyChange(() => Fabricant);
             }
         }

         public string Referencia
         {
             get { return ElementModel.Referencia; }
             set
             {
                 ElementModel.Referencia = value;
                 NotifyOfPropertyChange(() => Referencia);
             }
         }

         public string Carrer
         {
             get { return ElementModel.Carrer; }
             set
             {
                 ElementModel.Carrer = value;
                 NotifyOfPropertyChange(() => Carrer);
                 NotifyOfPropertyChange(() => CanSave);

                 if (string.IsNullOrEmpty(value))
                 {
                     LastError = "Es requereix el Carrer";
                     _dataErrors["Carrer"] = LastError;
                 }
                 else
                 {
                     if (_dataErrors.ContainsKey("Carrer"))
                         _dataErrors.Remove("Carrer");
                 }
             }
         }

         public string Numero
         {
             get { return ElementModel.NumeroAproximat; }
             set
             {
                 ElementModel.NumeroAproximat = value;
                 NotifyOfPropertyChange(() => Numero);
             }
         }

         public string CodiPostal
         {
             get { return ElementModel.CodiPostal; }
             set
             {
                 ElementModel.CodiPostal = value;
                 NotifyOfPropertyChange(() => CodiPostal);
                 NotifyOfPropertyChange(() => CanSave);

                 if (string.IsNullOrEmpty(value))
                 {
                     LastError = "Es requereix el CodiPostal";
                     _dataErrors["CodiPostal"] = LastError;
                 }
                 else
                 {
                     if (_dataErrors.ContainsKey("CodiPostal"))
                         _dataErrors.Remove("CodiPostal");
                 }
             }
         }

         public int Sector
         {
             get { return ElementModel.Id_sectorXarxa; }
             set
             {
                 ElementModel.Id_sectorXarxa = value;
                 NotifyOfPropertyChange(() => Sector);
                 NotifyOfPropertyChange(() => CanSave);

                 if (value == 0)
                 {
                     LastError = "Es requereix el Sector";
                     _dataErrors["Sector"] = LastError;
                 }
                 else
                 {
                     if (_dataErrors.ContainsKey("Sector"))
                         _dataErrors.Remove("Sector");
                 }
             }
         }

         public string Latitud
         {
             get { return ElementModel.PosicioLatitud; }
             set
             {
                 ElementModel.PosicioLatitud = value;
                 NotifyOfPropertyChange(() => Latitud);
             }
         }

         public string Longitud
         {
             get { return ElementModel.PosicioLongitud; }
             set
             {
                 ElementModel.PosicioLongitud = value;
                 NotifyOfPropertyChange(() => Longitud);
             }
         }

         public string Notes
         {
             get { return ElementModel.NotesPosicio; }
             set
             {
                 ElementModel.NotesPosicio = value;
                 NotifyOfPropertyChange(() => Notes);
             }
         }

         # endregion

         # region Commands

         public void UpdateById()
         {
             if (Id > 0)
             {
                 ElementModel = client.GetElementById(Id);
                 NotifyOfPropertyChange(string.Empty);
             }
         }

         public void New()
         {
             events.Publish(new CommandEvent { CommandToExecute = "ShowNewElement" });
         }

         public bool CanSave
         {
             get
             {
                 bool isValid = !(Tipus < 1 || String.IsNullOrEmpty(Carrer) || String.IsNullOrEmpty(CodiPostal)
                                  || Sector < 1 );

                 return isValid;
             }
         }

         public void Save()
         {
             if (ElementModel.Id == 0)
             {
                 if (client.AddElement(ElementModel, CacheMain.CurrentUserId))
                 {
                     MessageBox.Show("Element enregistrat correctament");
                     New();
                 }                     
                 else
                 {
                     MessageBox.Show("Error enregistrant l'Element");
                 }                     
             }
             else if (ElementModel.Id > 0)
             {
                 if (client.UpdateElement(ElementModel, CacheMain.CurrentUserId))
                 {
                     MessageBox.Show("Element actualitzat correctament");
                     New();
                 }
                 else
                 {
                     MessageBox.Show("Error actualitzant l'Element");
                 }
             }
             NotifyOfPropertyChange(LastError);
             NotifyOfPropertyChange(FormErrors);
         }

         public void Delete()
         {
             if (Id > 0)
             {
                 if (client.DeleteElement(Id, CacheMain.CurrentUserId))
                 {
                     MessageBox.Show("Element eliminat correctament");
                     // Si l'esborrat ha sigut correcte s'obre un nou formulari.
                     New();
                 }
                 else
                 {
                     MessageBox.Show("Error eliminant l'Element");
                 }
             }
         }

         public void Cancel()
         {
             New();
         }
         # endregion

         # region Events
         public void Handle(CommandEvent pCommandEvent)
         {
             if (pCommandEvent.CommandToExecute.Equals("DeleteElement"))
             {
                 Delete();
             }
         }
         # endregion

         #region IDataErrorInfo Members

         private string _dataError = string.Empty;

         public string Error
         {
             get { return _dataError; }
         }

         protected Dictionary<string, string> _dataErrors =
                        new Dictionary<string, string>();

         public string this[string columnName]
         {
             get
             {
                 //Activa o desactiva el missatge general d'errors
                 if (_dataErrors.Count != 0)
                 {
                     FormErrors = "Visible";
                     NotifyOfPropertyChange(() => FormErrors);
                 }
                 else
                 {
                     FormErrors = "Collapsed";
                     NotifyOfPropertyChange(() => FormErrors);
                 }
                 
                 if (_dataErrors.ContainsKey(columnName))
                 {
                     return _dataErrors[columnName];
                 }

                 else
                     return null;
             }
         }

         #endregion
    }
}
