﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.PresentationWPF.ElementSearch
{
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.ComponentModel.Composition;
    using Caliburn.Micro;
    using TFC.PublicLighting.PresentationWPF.Cache;
    using TFC.PublicLighting.PresentationWPF.Events;
    using TFC.PublicLighting.PresentationWPF.PLManagementServiceReference;

    [Export(typeof(ElementSearchViewModel))]
    public class ElementSearchViewModel : Screen
    {
        private readonly IWindowManager windowManager;
        private IEventAggregator events;

        private List<SectorDTO> _sectorsList;
        private List<ElementDTO> _elementsList;
        private ElementDTO _selectedElement;

        // SERVEI CLIENT per interactuar amb els serveis distribuits.
        PLManagementServiceClient client = new PLManagementServiceClient();

        // MODEL de dades del ViewModel
        private ElementDTO _searchElementModel = new ElementDTO() { Id_sectorXarxa = 0};

        [ImportingConstructor]
        public ElementSearchViewModel(IEventAggregator pEvents, IWindowManager pWindowManager)
        {
            // Obté l'agregador d'esdeveniments a través del constructor i subscriu aquest ViewModel
            // perquè pugui escoltar els missatges d'esdeveniments.
            events = pEvents;
            events.Subscribe(this);

            windowManager = pWindowManager;

            SectorsList = client.GetAllSectors();
            SectorDTO noSector = new SectorDTO() { IdSector = 0, NomBarri = "-" };
            SectorsList.Insert(0, noSector);

            ElementsList = client.GetAllElements();
        }

        # region Properties

        public ElementDTO ElementSearchModel
        {
            get { return _searchElementModel; }
            set
            {
                _searchElementModel = value;
                NotifyOfPropertyChange(() => ElementSearchModel);
                NotifyOfPropertyChange(string.Empty);
            }
        }

        /// <summary>
        /// Element Seleccionat
        /// </summary>
        public ElementDTO SelectedElement
        {
            get { return _selectedElement; }
            set
            {
                CacheMain.CurrentElement = value;
                CacheMain.IsNewElement = false;
                _selectedElement = value;
                NotifyOfPropertyChange(() => SelectedElement);
            }
        }

        public List<SectorDTO> SectorsList
        {
            get
            {
                return _sectorsList;
            }
            set
            {
                _sectorsList = value;
                NotifyOfPropertyChange(() => SectorsList);
            }
        }

        public List<ElementDTO> ElementsList
        {
            get
            {
                return _elementsList;
            }
            set
            {
                _elementsList = value;
                NotifyOfPropertyChange(() => ElementsList);
            }
        }

        public int Id
        {
            get { return ElementSearchModel.Id; }
            set
            {
                ElementSearchModel.Id = value;
                NotifyOfPropertyChange(() => Id);
            }
        }

        public string Matricula
        {
            get { return ElementSearchModel.Matricula; }
            set
            {
                ElementSearchModel.Matricula = value;
                NotifyOfPropertyChange(() => Matricula);
            }
        }

        public string Carrer
        {
            get { return ElementSearchModel.Carrer; }
            set
            {
                ElementSearchModel.Carrer = value;
                NotifyOfPropertyChange(() => Carrer);
            }
        }

        public int Sector
        {
            get { return ElementSearchModel.Id_sectorXarxa; }
            set
            {
                int intValue = value;
                ElementSearchModel.Id_sectorXarxa = intValue;
                NotifyOfPropertyChange(() => Sector);
            }
        }

        # endregion

        # region Commands

        // Actualitza la llista de resultats segons el filtre actual.
        public void Update()
        {
            ElementsList = client.GetElementsByFilter(ElementSearchModel);
        }

        // Enregistra a la memòria caché l'element seleccionat. 
        // Obre el formulari amb les seves dades.
        public void SelectElement()
        {
            CacheMain.IsNewElement = false;
            events.Publish(new CommandEvent { CommandToExecute = "ShowElement" });
        }

        # endregion
    }
}

