﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.PresentationWPF.ElementsManagement
{
    using System.ComponentModel.Composition;
    using System.Windows;

    using Caliburn.Micro;
    using TFC.PublicLighting.PresentationWPF.Element;
    using TFC.PublicLighting.PresentationWPF.ElementSearch;
    using TFC.PublicLighting.PresentationWPF.Events;
    using TFC.PublicLighting.PresentationWPF.Cache;

    [Export(typeof(ElementsManagementViewModel))]
    public class ElementsManagementViewModel : Conductor<object>, IHandle<CommandEvent>
    {
        private readonly IWindowManager windowManager;
        private IEventAggregator events;

        private string _controlsAdministrador;

        [ImportingConstructor]
        public ElementsManagementViewModel(IEventAggregator pEvents, IWindowManager pWindowManager)
        {
            // Obté l'agregador d'esdeveniments a través del constructor i subscriu aquest ViewModel
            // perquè pugui escoltar els missatges d'esdeveniments.
            events = pEvents;
            events.Subscribe(this);

            windowManager = pWindowManager;

            // Comproba quin tipus d'usuari és per activar els controls corresponents.
            if (CacheMain.IsAdministradorOn)
            {
                ControlsAdministrador = "Visible";
            }
            else
            {
                ControlsAdministrador = "Collapsed";
            }

            ShowElement();
        }

        public string ControlsAdministrador
        {
            get { return _controlsAdministrador; }
            set
            {
                _controlsAdministrador = value;
                NotifyOfPropertyChange(() => ControlsAdministrador);
            }
        }

        # region Commands
        public void ShowSearchElement()
        {
            ActivateItem(new ElementSearchViewModel(events, windowManager));
        }

        public void DeleteElement()
        {
            string message = "Estàs segur que vols esborrar l'Element actual?";
            string caption = "Confirmació";
            MessageBoxButton buttons = MessageBoxButton.YesNo;
            MessageBoxImage icon = MessageBoxImage.Question;
            if (MessageBox.Show(message, caption, buttons, icon) == MessageBoxResult.Yes)
            {
                // OK codi
                events.Publish(new CommandEvent { CommandToExecute = "DeleteElement" });
            }            
        }

        public void ShowElement()
        {
            ActivateItem(new ElementViewModel(events, windowManager));
        }

        public void ShowNewElement()
        {
            CacheMain.IsNewElement = true;
            ActivateItem(new ElementViewModel(events, windowManager));
        }
        # endregion

        # region Events
        public void Handle(CommandEvent pCommandEvent)
        {
            if (pCommandEvent.CommandToExecute.Equals("ShowNewElement"))
            {
                ShowNewElement();
            }
            else if (pCommandEvent.CommandToExecute.Equals("ShowElement"))
            {
                ShowElement();
            }
        }
        # endregion
    }
}
