﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.PresentationWPF.IncidenciaSearch
{
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.ComponentModel.Composition;
    using Caliburn.Micro;
    using TFC.PublicLighting.PresentationWPF.Cache;
    using TFC.PublicLighting.PresentationWPF.Events;
    using TFC.PublicLighting.PresentationWPF.PLManagementServiceReference;

    [Export(typeof(IncidenciaSearchViewModel))]
    public class IncidenciaSearchViewModel : Screen
    {
        private readonly IWindowManager windowManager;
        private IEventAggregator events;

        private List<EstatIncidenciaDTO> _estatsList;
        private List<IncidenciaDTO> _incidenciesList;
        private IncidenciaDTO _selectedIncidencia;

        // SERVEI CLIENT per interactuar amb els serveis distribuits.
        PLManagementServiceClient client = new PLManagementServiceClient();

        // MODEL de dades del ViewModel
        private IncidenciaDTO _searchIncidenciaModel = new IncidenciaDTO();

        [ImportingConstructor]
        public IncidenciaSearchViewModel(IEventAggregator pEvents, IWindowManager pWindowManager)
        {
            // Obté l'agregador d'esdeveniments a través del constructor i subscriu aquest ViewModel
            // perquè pugui escoltar els missatges d'esdeveniments.
            events = pEvents;
            events.Subscribe(this);

            windowManager = pWindowManager;

            EstatsList = client.GetAllEstatsIncidencia();
            IncidenciesList = client.GetAllIncidencies();
        }

        # region Properties

        public IncidenciaDTO IncidenciaSearchModel
        {
            get { return _searchIncidenciaModel; }
            set
            {
                _searchIncidenciaModel = value;
                NotifyOfPropertyChange(() => IncidenciaSearchModel);
                NotifyOfPropertyChange(string.Empty);
            }
        }

        /// <summary>
        /// Incidencia Seleccionada
        /// </summary>
        public IncidenciaDTO SelectedIncidencia
        {
            get { return _selectedIncidencia; }
            set
            {
                CacheMain.CurrentIncidencia = value;
                CacheMain.IsNewIncidencia = false;
                _selectedIncidencia = value;
                NotifyOfPropertyChange(() => SelectedIncidencia);
            }
        }

        public List<IncidenciaDTO> IncidenciesList
        {
            get
            {
                return _incidenciesList;
            }
            set
            {
                _incidenciesList = value;
                NotifyOfPropertyChange(() => IncidenciesList);
            }
        }

        public List<EstatIncidenciaDTO> EstatsList
        {
            get
            {
                return _estatsList;
            }
            set
            {
                _estatsList = value;
                NotifyOfPropertyChange(() => EstatsList);
            }
        }

        /// <summary>
        /// Identificació de la Incidència a la BBDD
        /// </summary>
        public int IDIncidencia
        {
            get { return IncidenciaSearchModel.Id_incidencia; }
            set
            {
                IncidenciaSearchModel.Id_incidencia = value;
                NotifyOfPropertyChange(() => IDIncidencia);
            }
        }

        public int IDElement
        {
            get { return IncidenciaSearchModel.Id_element; }
            set
            {
                IncidenciaSearchModel.Id_element = value;
                NotifyOfPropertyChange(() => IDElement);
            }
        }
        
        # endregion

        # region Commands

        public void Update()
        {
            IncidenciesList = client.GetIncidenciesByFilter(IncidenciaSearchModel);
        }

        public void SelectIncidencia()
        {
            CacheMain.IsNewIncidencia = false;
            events.Publish(new CommandEvent { CommandToExecute = "ShowIncidencia" });
        }

        # endregion
    }
}


