﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.PresentationWPF.IncidenciesManagement
{
    using System.ComponentModel.Composition;
    using Caliburn.Micro;
    using TFC.PublicLighting.PresentationWPF.Cache;
    using TFC.PublicLighting.PresentationWPF.Events;
    using TFC.PublicLighting.PresentationWPF.Incidencia;
    using TFC.PublicLighting.PresentationWPF.IncidenciaSearch;

    [Export(typeof(IncidenciesManagementViewModel))]
    public class IncidenciesManagementViewModel : Conductor<object>, IHandle<CommandEvent>
    {
        private readonly IWindowManager windowManager;
        private IEventAggregator events;

        [ImportingConstructor]
        public IncidenciesManagementViewModel(IEventAggregator pEvents, IWindowManager pWindowManager)
        {
            // Obté l'agregador d'esdeveniments a través del constructor i subscriu aquest ViewModel
            // perquè pugui escoltar els missatges d'esdeveniments.
            events = pEvents;
            events.Subscribe(this);

            windowManager = pWindowManager;

            ShowIncidencia();
        }

        # region Commands
        public void ShowSearchIncidencia()
        {
            ActivateItem(new IncidenciaSearchViewModel(events, windowManager));
        }

        public void DeleteIncidencia()
        {
            events.Publish(new CommandEvent { CommandToExecute = "DeleteIncidencia" });
        }

        public void ShowIncidencia()
        {
            ActivateItem(new IncidenciaViewModel(events, windowManager));
        }
        
        public void ShowNewIncidencia()
        {
            CacheMain.IsNewIncidencia = true;            
            ActivateItem(new IncidenciaViewModel(events, windowManager));
        }

        # endregion

        # region Events
        public void Handle(CommandEvent pCommandEvent)
        {
            if (pCommandEvent.CommandToExecute.Equals("ShowNewIncidencia"))
            {
                ShowNewIncidencia();
            }
            else if (pCommandEvent.CommandToExecute.Equals("ShowIncidencia"))
            {
                ShowIncidencia();
            }
        }
           # endregion
    }
}


