﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.PresentationWPF.Loggin
{
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.ComponentModel.Composition;
    using System.Windows;
    using System.Windows.Controls;
    using Caliburn.Micro;
    using TFC.PublicLighting.PresentationWPF.Cache;
    using TFC.PublicLighting.PresentationWPF.Events;
    using TFC.PublicLighting.PresentationWPF.PLManagementServiceReference;

    [Export(typeof(LogginViewModel))]
    public class LogginViewModel : Conductor<object>, IDataErrorInfo
    {
        private IEventAggregator events;

        // SERVEI CLIENT per interactuar amb els serveis distribuits.
        PLManagementServiceClient client = new PLManagementServiceClient();

        private UsuariDTO _loggedUser;
        private string _usuari;
        private PasswordBox passwordBox;
        
        public LogginViewModel(IEventAggregator pEvents)
        {
            // Obté l'agregador d'esdeveniments a través del constructor perquè pugui publicar esdeveniments.
            events = pEvents;

            // SERVEI CLIENT per interactuar amb els serveis distribuits.
            PLManagementServiceClient client = new PLManagementServiceClient();
        }

        # region Properties
        public string Usuari
        {
            get { return _usuari; }
            set
            {
                _usuari = value;
                NotifyOfPropertyChange(() => Usuari);

                if (string.IsNullOrEmpty(value))
                {
                    _dataErrors["Usuari"] = "Usuari";
                }
                else
                {
                    if (_dataErrors.ContainsKey("Usuari"))
                        _dataErrors.Remove("Usuari");
                }
            }
        }

        # endregion

        # region Commands
        public void Cancel() 
        {
            events.Publish(new LoggingEvent { LoggedUser = "ShutDown" });
        }
                
        public void Loggin()
        {
           // Validar password (entre 4 i 12 caràcters)
            bool isValidPassword = !(string.IsNullOrEmpty(passwordBox.Password) || passwordBox.Password.Length > 12 || passwordBox.Password.Length < 4);

            if (string.IsNullOrEmpty(Usuari))
            {
                MessageBox.Show("Es requereix un nom d'usuari");
            }
            else if (!isValidPassword)
            {
                MessageBox.Show("Es requereix una contrasenya entre 4 i 12 caràcters");
            }
            else
            {
               // Crida al servei per validar la identificació
                _loggedUser = client.GetLoggedUser(Usuari, passwordBox.Password);

                if (_loggedUser == null) 
                {
                    passwordBox.Password = string.Empty;
                    Usuari = string.Empty;
                    MessageBox.Show("Usuari no identificat");                    
                }
                else if (_loggedUser.IdTipusUsuari == "Tecnic")
                {
                    passwordBox.Password = string.Empty;
                    Usuari = string.Empty;
                    MessageBox.Show("Usuari no autoritzat");  
                }
                else
                {
                    if (_loggedUser.IdTipusUsuari == "Administrador")
                    {
                        CacheMain.IsAdministradorOn = true;
                        CacheMain.CurrentUserId = _loggedUser.Id;
                        MessageBox.Show("Benvingut Administrador: " + _loggedUser.Nom + " " + _loggedUser.Cognom1);
                        events.Publish(new LoggingEvent { LoggedUser = "Administrador" });
                    }
                    else if (_loggedUser.IdTipusUsuari == "Supervisor")
                    {
                        CacheMain.IsAdministradorOn = false;
                        CacheMain.CurrentUserId = _loggedUser.Id;
                        MessageBox.Show("Benvingut Supervisor: " + _loggedUser.Nom + " " + _loggedUser.Cognom1);
                        events.Publish(new LoggingEvent { LoggedUser = "Supervisor" });
                    }
                }
            }                         
        }

        # endregion

        // Les propietats del control PasswordBox no es poden enllaçar al ViewModel.
        // Obté el View des de el ViewModel per poder accedir a la propietat Password del PasswordBox
        protected override void OnViewAttached(object view, object context)
        {
            base.OnViewAttached(view, context);

            var frameworkElement = view as FrameworkElement;
            passwordBox = frameworkElement.FindName("Clau") as PasswordBox;
        }

        #region IDataErrorInfo Members

        private string _dataError = string.Empty;

        public string Error
        {
            get { return _dataError; }
        }

        protected Dictionary<string, string> _dataErrors =
                       new Dictionary<string, string>();

        public string this[string columnName]
        {
            get
            {
                if (_dataErrors.ContainsKey(columnName))
                {
                    return _dataErrors[columnName];
                }

                else
                    return null;
            }
        }

        #endregion
    }
}