﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.PresentationWPF.MainAdministrador
{
    using System.ComponentModel.Composition;
    using Caliburn.Micro;
    using TFC.PublicLighting.PresentationWPF.ElementsManagement;
    using TFC.PublicLighting.PresentationWPF.TecnicsManagement;
    using TFC.PublicLighting.PresentationWPF.SupervisorManagement;
    using TFC.PublicLighting.PresentationWPF.Events;
    using TFC.PublicLighting.PresentationWPF.Cache;

    [Export(typeof(MainAdminViewModel))]
    public class MainAdminViewModel : Conductor<object>
    {
        private readonly IWindowManager windowManager;
        private IEventAggregator events;

        [ImportingConstructor]
        public MainAdminViewModel(IEventAggregator pEvents, IWindowManager pWindowManager)
        {
            // Get the event aggregator through the constructor and subscribe this ViewModel so it can listen for Event messages.
            events = pEvents;
            windowManager = pWindowManager;

            CleanCache();

            ShowTecnic();
        }

        # region Commands

        public void ShowSupervisor()
        {
            ActivateItem(new SupervisorManagementViewModel(events, windowManager));
        }

        public void ShowTecnic()
        {
            ActivateItem(new TecnicsManagementViewModel(events, windowManager));
        }

        public void ShowElement()
        {
            ActivateItem(new ElementsManagementViewModel(events, windowManager));
        }

        public void ShowLoggin()
        {
            events.Publish(new LoggingEvent { LoggedUser = "ShowLoggin" });
        }

        public void ShutDown()
        {
            events.Publish(new LoggingEvent { LoggedUser = "ShutDown" });
        }

        # endregion

        private void CleanCache()
        {
            CacheMain.IsNewTecnic = true;
            CacheMain.IsNewElement = true;
            CacheMain.IsNewIncidencia = true;
            CacheMain.IsNewOrdre = true;

        }
    }
}