﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.PresentationWPF.MainSupervisor
{
    using System.ComponentModel.Composition;
    using Caliburn.Micro;
    using TFC.PublicLighting.PresentationWPF.ElementSearch;
    using TFC.PublicLighting.PresentationWPF.Events;
    using TFC.PublicLighting.PresentationWPF.IncidenciesManagement;
    using TFC.PublicLighting.PresentationWPF.TecnicsManagement;
    using TFC.PublicLighting.PresentationWPF.ElementsManagement;
    using TFC.PublicLighting.PresentationWPF.OrdresManagement;
    using TFC.PublicLighting.PresentationWPF.TecnicSearch;
    using TFC.PublicLighting.PresentationWPF.Cache;

    [Export(typeof(MainSuperViewModel))]
    public class MainSuperViewModel : Conductor<object>
    {
        private readonly IWindowManager windowManager;
        private IEventAggregator events;

        [ImportingConstructor]
        public MainSuperViewModel(IEventAggregator pEvents, IWindowManager pWindowManager)
        {
            // Obté l'agregador d'esdeveniments a través del constructor.
            events = pEvents;
            windowManager = pWindowManager;

            CleanCache();

            ShowIncidencia();
        }

        # region Commands

        public void ShowTecnic()
        {
            ActivateItem(new TecnicsManagementViewModel(events, windowManager));
        }

        public void ShowElement()
        {
            ActivateItem(new ElementsManagementViewModel(events, windowManager));
        }

        public void ShowOrdre()
        {
            ActivateItem(new OrdresManagementViewModel(events, windowManager));
        }

        public void ShowIncidencia()
        {
            ActivateItem(new IncidenciesManagementViewModel(events, windowManager));
        }

        public void ShowLoggin()
        {
            events.Publish(new LoggingEvent { LoggedUser = "ShowLoggin" });
        }

        public void ShutDown()
        {
            events.Publish(new LoggingEvent { LoggedUser = "ShutDown" });
        }

        # endregion

        private void CleanCache()
        {
            CacheMain.IsNewTecnic = true;
            CacheMain.IsNewElement = true;
            CacheMain.IsNewIncidencia = true;
            CacheMain.IsNewOrdre = true;

        }
    }
}
