﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.PresentationWPF.OrdreSearch
{
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.ComponentModel.Composition;
    using Caliburn.Micro;
    using TFC.PublicLighting.PresentationWPF.Cache;
    using TFC.PublicLighting.PresentationWPF.Events;
    using TFC.PublicLighting.PresentationWPF.PLManagementServiceReference;

    [Export(typeof(OrdreSearchViewModel))]
    public class OrdreSearchViewModel : Screen
    {
        private readonly IWindowManager windowManager;
        private IEventAggregator events;

        private List<EstatOrdreDTO> _estatsList;
        private List<OrdreDTO> _ordresList;
        private OrdreDTO _selectedOrdre;

        // SERVEI CLIENT per interactuar amb els serveis distribuits.
        PLManagementServiceClient client = new PLManagementServiceClient();

        // MODEL de dades del ViewModel
        private OrdreDTO _searchOrdreModel = new OrdreDTO();

        [ImportingConstructor]
        public OrdreSearchViewModel(IEventAggregator pEvents, IWindowManager pWindowManager)
        {
            // Obté l'agregador d'esdeveniments a través del constructor i subscriu aquest ViewModel
            // perquè pugui escoltar els missatges d'esdeveniments.
            events = pEvents;
            events.Subscribe(this);

            windowManager = pWindowManager;

            EstatsList = client.GetAllEstatsOrdre();
            OrdresList = client.GetAllOrdres();
        }

        # region Properties

        public OrdreDTO OrdreSearchModel
        {
            get { return _searchOrdreModel; }
            set
            {
                _searchOrdreModel = value;
                NotifyOfPropertyChange(() => OrdreSearchModel);
                NotifyOfPropertyChange(string.Empty);
            }
        }

        /// <summary>
        /// Ordre Seleccionada
        /// </summary>
        public OrdreDTO SelectedOrdre
        {
            get { return _selectedOrdre; }
            set
            {
                CacheMain.CurrentOrdre = value;
                CacheMain.IsNewOrdre = false;
                _selectedOrdre = value;
                NotifyOfPropertyChange(() => SelectedOrdre);
            }
        }
        
        public List<OrdreDTO> OrdresList
        {
            get
            {
                return _ordresList;
            }
            set
            {
                _ordresList = value;
                NotifyOfPropertyChange(() => OrdresList);
            }
        }

        public List<EstatOrdreDTO> EstatsList
        {
            get
            {
                return _estatsList;
            }
            set
            {
                _estatsList = value;
                NotifyOfPropertyChange(() => EstatsList);
            }
        }

        /// <summary>
        /// Identificació de la Incidència a la BBDD
        /// </summary>
        public int IDOrdre
        {
            get { return OrdreSearchModel.Id_ordreReparacio; }
            set
            {
                OrdreSearchModel.Id_ordreReparacio = value;
                NotifyOfPropertyChange(() => IDOrdre);
            }
        }

        public int IDTecnic
        {
            get { return OrdreSearchModel.Id_tecnic; }
            set
            {
                OrdreSearchModel.Id_tecnic = value;
                NotifyOfPropertyChange(() => IDTecnic);
            }
        }

        public int Estat
        {
            get { return OrdreSearchModel.Id_estatOrdre; }
            set
            {
                OrdreSearchModel.Id_estatOrdre = value;
                NotifyOfPropertyChange(() => Estat);
            }
        }

        # endregion

        # region Commands

        public void Update()
        {
            OrdresList = client.GetOrdresByFilter(OrdreSearchModel);
        }

        public void SelectOrdre()
        {
            CacheMain.IsNewOrdre = false;
            events.Publish(new CommandEvent { CommandToExecute = "ShowOrdre" });
        }

        # endregion
    }
}



