﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.PresentationWPF.OrdresManagement
{
    using System.ComponentModel.Composition;
    using Caliburn.Micro;
    using TFC.PublicLighting.PresentationWPF.Cache;
    using TFC.PublicLighting.PresentationWPF.Events;
    using TFC.PublicLighting.PresentationWPF.Ordre;
    using TFC.PublicLighting.PresentationWPF.OrdreSearch;

    [Export(typeof(OrdresManagementViewModel))]
    public class OrdresManagementViewModel : Conductor<object>, IHandle<CommandEvent>
    {
        private readonly IWindowManager windowManager;
        private IEventAggregator events;

        [ImportingConstructor]
        public OrdresManagementViewModel(IEventAggregator pEvents, IWindowManager pWindowManager)
        {
            // Obté l'agregador d'esdeveniments a través del constructor i subscriu aquest ViewModel
            // perquè pugui escoltar els missatges d'esdeveniments.
            events = pEvents;
            events.Subscribe(this);

            windowManager = pWindowManager;

            ShowOrdre();
        }

        # region Commands
        public void ShowSearchOrdre()
        {
            ActivateItem(new OrdreSearchViewModel(events, windowManager));
        }

        public void DeleteOrdre()
        {
            events.Publish(new CommandEvent { CommandToExecute = "DeleteOrdre" });
        }

        public void ShowOrdre()
        {
            ActivateItem(new OrdreViewModel(events, windowManager));
        }
                
        public void ShowNewOrdre()
        {
            CacheMain.IsNewOrdre = true;
            ActivateItem(new OrdreViewModel(events, windowManager));
        }
        # endregion

        # region Events
        public void Handle(CommandEvent pCommandEvent)
        {
            if (pCommandEvent.CommandToExecute.Equals("ShowNewOrdre"))
            {
                ShowNewOrdre();
            }
            else if (pCommandEvent.CommandToExecute.Equals("ShowOrdre"))
            {
                ShowOrdre();
            }                            
        }
        # endregion
    }
}

