﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.PresentationWPF.Shell
{
    using System.ComponentModel.Composition;
    using Caliburn.Micro;
    using TFC.PublicLighting.PresentationWPF.Events;
    using TFC.PublicLighting.PresentationWPF.Loggin;
    using TFC.PublicLighting.PresentationWPF.MainAdministrador;
    using TFC.PublicLighting.PresentationWPF.MainSupervisor;

    [Export(typeof(ShellViewModel))]
    public class ShellViewModel : Conductor<object>, IHandle<LoggingEvent>, IHaveDisplayName
    {
        private readonly IWindowManager windowManager;
        private IEventAggregator events;
        private string _displayName = "Sistema d'Incidències Enllumenat Públic";

        [ImportingConstructor]
        public ShellViewModel(IEventAggregator pEvents, IWindowManager pWindowManager)
        {
            events = pEvents;
            windowManager = pWindowManager;

            events.Subscribe(this);

            ShowLoggin();
        }

        public override string DisplayName
        {
            get { return _displayName; }
            set { _displayName = value; }
        }

        public void ShowAdministrador()
        {
            ActivateItem(new MainAdminViewModel(events, windowManager));
        }

        public void ShowSupervisor()
        {
            ActivateItem(new MainSuperViewModel(events, windowManager));
        }

        public void ShowLoggin()
        {
            ActivateItem(new LogginViewModel(events));
        }


        // This method is called after a Event message is published from somewhere else in the application.
        public void Handle(LoggingEvent loggingEvent)
        {
            if (loggingEvent.LoggedUser == "Administrador")
            {
                ShowAdministrador();
            }
            else if (loggingEvent.LoggedUser == "Supervisor")
            {
                ShowSupervisor();
            }
            else if (loggingEvent.LoggedUser == "ShowLoggin")
            {
                ShowLoggin();
            }
            else if (loggingEvent.LoggedUser == "ShutDown")
            {
                this.TryClose();
            }
        }
    }
}

