﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.PresentationWPF.Supervisor
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.ComponentModel.Composition;
    using System.Text.RegularExpressions;
    using Caliburn.Micro;
    using TFC.PublicLighting.PresentationWPF.Cache;
    using TFC.PublicLighting.PresentationWPF.Events;
    using TFC.PublicLighting.PresentationWPF.PLManagementServiceReference;

    [Export(typeof(SupervisorViewModel))]
    public class SupervisorViewModel : Screen, IDataErrorInfo, IHandle<CommandEvent>
    {
        private readonly IWindowManager windowManager;
        private IEventAggregator events;

        private string _formErrors;
        private string _lastError = "Formulari incomplert o amb errors";

        // SERVEI CLIENT per interactuar amb els serveis distribuits.
        PLManagementServiceClient client = new PLManagementServiceClient();

        // MODEL de dades del ViewModel
        private UsuariDTO _supervisorModel;

        [ImportingConstructor]
        public SupervisorViewModel(IEventAggregator pEvents, IWindowManager pWindowManager, bool isNewSupervisor)
        {
            // Obté l'agregador d'esdeveniments a través del constructor i subscriu aquest ViewModel
            // perquè pugui escoltar els missatges d'esdeveniments.
            events = pEvents;
            events.Subscribe(this);

            windowManager = pWindowManager;

            if (isNewSupervisor)
            {
                SupervisorModel = new UsuariDTO() { Id = 0, TipusDocId = "DNI", DataNeixement = DateTime.Now };
            }
            else
            {
                SupervisorModel = client.GetSupervisor();
                if (SupervisorModel == null)
                    SupervisorModel = new UsuariDTO() { Id = 0, TipusDocId = "DNI", DataNeixement = DateTime.Now }; 
            }  
        }

        # region Error
        public string FormErrors
        {
            get { return _formErrors; }
            set
            {
                _formErrors = value;
                NotifyOfPropertyChange(() => FormErrors);
            }
        }

        public string LastError
        {
            get { return _lastError; }
            set
            {
                _lastError = value;
                NotifyOfPropertyChange(() => LastError);
            }
        }
        # endregion

        # region Properties

        public UsuariDTO SupervisorModel
        {
            get { return _supervisorModel; }
            set
            {
                _supervisorModel = value;
                NotifyOfPropertyChange(() => SupervisorModel);
                FormErrors = "Collapsed";
                NotifyOfPropertyChange(string.Empty);
            }
        }

        /// <summary>
        /// Identificació del Tècnic a la BBDD
        /// </summary>
        public int Id
        {
            get { return SupervisorModel.Id; }
            set
            {
                SupervisorModel.Id = value;
                NotifyOfPropertyChange(() => Id);

                if (value < 0)
                {
                    LastError = "L'Id és incorrecte";
                    _dataErrors["Id"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("Id"))
                        _dataErrors.Remove("Id");
                }
                NotifyOfPropertyChange(() => CanSave);
            }
        }

        public string Nom
        {
            get { return SupervisorModel.Nom; }
            set
            {
                SupervisorModel.Nom = value;
                NotifyOfPropertyChange(() => Nom);
                NotifyOfPropertyChange(() => CanSave);

                if (string.IsNullOrEmpty(value))
                {
                    LastError = "Es requereix el Nom";
                    _dataErrors["Nom"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("Nom"))
                        _dataErrors.Remove("Nom");
                }
            }
        }

        public string Cognom1
        {
            get { return SupervisorModel.Cognom1; }
            set
            {
                SupervisorModel.Cognom1 = value;
                NotifyOfPropertyChange(() => Cognom1);
                NotifyOfPropertyChange(() => CanSave);

                if (string.IsNullOrEmpty(value))
                {
                    LastError = "Es requereix el Primer Cognom";
                    _dataErrors["Cognom1"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("Cognom1"))
                        _dataErrors.Remove("Cognom1");
                }
            }
        }

        public string Cognom2
        {
            get { return SupervisorModel.Cognom2; }
            set
            {
                SupervisorModel.Cognom2 = value;
                NotifyOfPropertyChange(() => Cognom2);
            }
        }

        public List<string> TipusDocIdList
        {
            get
            {
                return new List<string> { "DNI", "NIE" };
            }
        }

        public string TipusDocId
        {
            get { return SupervisorModel.TipusDocId; }
            set
            {
                SupervisorModel.TipusDocId = value;
                NotifyOfPropertyChange(() => TipusDocId);
                NotifyOfPropertyChange(() => CanSave);

                if (string.IsNullOrEmpty(value))
                {
                    LastError = "Es requereix el tipus de Document d'Identitat";
                    _dataErrors["TipusDocId"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("TipusDocId"))
                        _dataErrors.Remove("TipusDocId");

                    if (!ValidateDocumentId(TipusDocId, DocumentId))
                    {
                        LastError = "Es requereix un " + TipusDocId + " vàlid";
                        _dataErrors["DocumentId"] = LastError;
                    }
                    else
                    {
                        if (_dataErrors.ContainsKey("DocumentId"))
                            _dataErrors.Remove("DocumentId");
                    }
                }
            }
        }

        public string DocumentId
        {
            get { return SupervisorModel.DocumentId; }
            set
            {
                SupervisorModel.DocumentId = value;
                NotifyOfPropertyChange(() => DocumentId);
                NotifyOfPropertyChange(() => CanSave);

                if (!ValidateDocumentId(TipusDocId, DocumentId))
                {
                    LastError = "Es requereix un " + TipusDocId + " vàlid";
                    _dataErrors["DocumentId"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("DocumentId"))
                        _dataErrors.Remove("DocumentId");
                }
            }
        }

        public DateTime? DataNeixement
        {
            get { return SupervisorModel.DataNeixement; }
            set
            {
                SupervisorModel.DataNeixement = value;
                NotifyOfPropertyChange(() => DataNeixement);
            }
        }

        public string Adreca
        {
            get { return SupervisorModel.Adreca; }
            set
            {
                SupervisorModel.Adreca = value;
                NotifyOfPropertyChange(() => Adreca);
            }
        }

        public string Poblacio
        {
            get { return SupervisorModel.Poblacio; }
            set
            {
                SupervisorModel.Poblacio = value;
                NotifyOfPropertyChange(() => Poblacio);
            }
        }

        public string CodiPostal
        {
            get { return SupervisorModel.CodiPostal; }
            set
            {
                SupervisorModel.CodiPostal = value;
                NotifyOfPropertyChange(() => CodiPostal);
                NotifyOfPropertyChange(() => CanSave);

                if (!ValidateCodiPostal(CodiPostal))
                {
                    LastError = "Codi Postal invalid";
                    _dataErrors["CodiPostal"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("CodiPostal"))
                        _dataErrors.Remove("CodiPostal");
                }
            }
        }

        public string Provincia
        {
            get { return SupervisorModel.Provincia; }
            set
            {
                SupervisorModel.Provincia = value;
                NotifyOfPropertyChange(() => Provincia);
            }
        }

        public string TelefonPrivat
        {
            get { return SupervisorModel.TelefonPrivat; }
            set
            {
                SupervisorModel.TelefonPrivat = value;
                NotifyOfPropertyChange(() => TelefonPrivat);
                NotifyOfPropertyChange(() => CanSave);

                if (!ValidateTelefon(TelefonPrivat))
                {
                    LastError = "Telèfon invalid";
                    _dataErrors["TelefonPrivat"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("TelefonPrivat"))
                        _dataErrors.Remove("TelefonPrivat");
                }
            }
        }

        public string TelefonOficina
        {
            get { return SupervisorModel.TelefonOficina; }
            set
            {
                SupervisorModel.TelefonOficina = value;
                NotifyOfPropertyChange(() => TelefonOficina);
                NotifyOfPropertyChange(() => CanSave);

                if (!ValidateTelOficina(value))
                {
                    LastError = "Es requereix un Telèfon d'oficina vàlid";
                    _dataErrors["TelefonOficina"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("TelefonOficina"))
                        _dataErrors.Remove("TelefonOficina");
                }
            }
        }

        public string MobilEmpresa
        {
            get { return SupervisorModel.MobilEmpresa; }
            set
            {
                SupervisorModel.MobilEmpresa = value;
                NotifyOfPropertyChange(() => MobilEmpresa);
                NotifyOfPropertyChange(() => CanSave);

                if (!ValidateMobilEmpresa(value))
                {
                    LastError = "Es requereix un Telef. mobil d'empresa vàlid";
                    _dataErrors["MobilEmpresa"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("MobilEmpresa"))
                        _dataErrors.Remove("MobilEmpresa");
                }
            }
        }

        public string Email
        {
            get { return SupervisorModel.Email; }
            set
            {
                SupervisorModel.Email = value;
                NotifyOfPropertyChange(() => Email);
                NotifyOfPropertyChange(() => CanSave);

                if (!ValidateEmail(Email))
                {
                    LastError = "Email incorrecte";
                    _dataErrors["Email"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("Email"))
                        _dataErrors.Remove("Email");
                }
            }
        }

        public string Usuari
        {
            get { return SupervisorModel.Usuari; }
            set
            {
                SupervisorModel.Usuari = value;
                NotifyOfPropertyChange(() => Usuari);
                NotifyOfPropertyChange(() => CanSave);

                if (!ValidateIdentificacio(Usuari))
                {
                    LastError = "Es requereix el nom d'Usuari entre 4 i 12 caràcters";
                    _dataErrors["Usuari"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("Usuari"))
                        _dataErrors.Remove("Usuari");
                }
            }
        }

        public string Contrasenya
        {
            get { return SupervisorModel.Contrasenya; }
            set
            {
                SupervisorModel.Contrasenya = value;
                NotifyOfPropertyChange(() => Contrasenya);
                NotifyOfPropertyChange(() => CanSave);

                if (!ValidateIdentificacio(Contrasenya))
                {
                    LastError = "Es requereix una Contrasenya entre 4 i 12 caràcters";
                    _dataErrors["Contrasenya"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("Contrasenya"))
                        _dataErrors.Remove("Contrasenya");
                }
            }
        }

        # endregion

        # region Commands

        public void New()
        {
            events.Publish(new CommandEvent { CommandToExecute = "ShowNewSupervisor" });
        }

        public bool CanSave
        {
            get 
            {
                bool isValid = !String.IsNullOrEmpty(Nom) && !String.IsNullOrEmpty(Cognom1) && !String.IsNullOrEmpty(TipusDocId)
                                && ValidateDocumentId(TipusDocId, DocumentId) && ValidateIdentificacio(Usuari) && ValidateIdentificacio(Contrasenya) 
                                && ValidateEmail(Email) && ValidateMobilEmpresa(MobilEmpresa) && ValidateTelefon(TelefonPrivat)
                                && ValidateTelOficina(TelefonOficina) && ValidateCodiPostal(CodiPostal);

                return isValid;
            }
        }

        public void Save()
        {
            if (SupervisorModel.Id == 0)
            {
                if (!client.AddSupervisor(SupervisorModel, CacheMain.CurrentUserId))
                {
                    LastError = "Error a la Base de Dades";
                    FormErrors = "Visible";
                }
            }
            else if (SupervisorModel.Id > 0)
            {
                if (!client.UpdateSupervisor(SupervisorModel, CacheMain.CurrentUserId))
                {
                    LastError = "Error a la Base de Dades";
                    FormErrors = "Visible";
                }
            }
            NotifyOfPropertyChange(LastError);
            NotifyOfPropertyChange(FormErrors);
        }

        public void Cancel()
        {
            events.Publish(new CommandEvent { CommandToExecute = "ShowSupervisor" });
        }

        # endregion

        # region Events
        public void Handle(CommandEvent pCommandEvent)
        {
            //if (pCommandEvent.CommandToExecute.Equals("DeleteSupervisor"))
            //{
            //    Delete();
            //}
        }
        # endregion

        #region IDataErrorInfo Members

        private string _dataError = string.Empty;

        public string Error
        {
            get { return _dataError; }
        }

        protected Dictionary<string, string> _dataErrors =
                       new Dictionary<string, string>();

        public string this[string columnName]
        {
            get
            {
                //Activa o desactiva el missatge general d'errors
                if (_dataErrors.Count != 0)
                {
                    FormErrors = "Visible";
                    NotifyOfPropertyChange(() => FormErrors);
                }
                else
                {
                    FormErrors = "Collapsed";
                    NotifyOfPropertyChange(() => FormErrors);
                }
                
                if (_dataErrors.ContainsKey(columnName))
                {
                    return _dataErrors[columnName];
                }

                else
                    return null;
            }
        }

        #endregion

        # region Auxiliar Methods

        private bool ValidateEmail(string email)
        {
            Regex emailRegex = new Regex(@"^([\w\.\-]+)@([\w\-]+)((\.(\w){2,3})+)$");
            bool isValid = string.IsNullOrEmpty(email) || emailRegex.Match(email).Success;
            return isValid;
        }

        private bool ValidateDni(string dni)
        {
            if (dni != null)
            {
                Regex dniRegex = new Regex(@"^[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][A-Z]$");
                bool isValid = dniRegex.Match(dni).Success;
                return isValid;
            }
            else return false;
        }

        private bool ValidateNie(string nie)
        {
            Regex nieRegex = new Regex(@"^[XYZ][0-9][0-9][0-9][0-9][0-9][0-9][0-9][A-Z]$");
            bool isValid = nieRegex.Match(nie).Success;
            return isValid;
        }

        private bool ValidateDocumentId(string tipus, string document)
        {
            if (tipus == "DNI")
            {
                return ValidateDni(document);
            }
            else if (tipus == "NIE")
            {
                return ValidateNie(document);
            }
            else return false;
        }

        private bool ValidateIdentificacio(string identificacio)
        {
            return !(string.IsNullOrEmpty(identificacio) || identificacio.Length < 4 || identificacio.Length > 12);
        }

        private bool ValidateCodiPostal(string codiPostal)
        {
            Regex nieRegex = new Regex(@"^[0-9][0-9][0-9][0-9][0-9]$");
            bool isValid = string.IsNullOrEmpty(codiPostal) || nieRegex.Match(codiPostal).Success;
            return isValid;
        }

        private bool ValidateTelefon(string codiPostal)
        {
            Regex telefRegex = new Regex(@"^[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]$");
            bool isValid = string.IsNullOrEmpty(codiPostal) || telefRegex.Match(codiPostal).Success;
            return isValid;
        }

        private bool ValidateTelOficina(string codiPostal)
        {
            Regex telefRegex = new Regex(@"^[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]$");
            bool isValid = telefRegex.Match(codiPostal).Success;
            return isValid;
        }

        private bool ValidateMobilEmpresa(string codiPostal)
        {
            Regex telefRegex = new Regex(@"^[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]$");
            bool isValid = telefRegex.Match(codiPostal).Success;
            return isValid;
        }

        # endregion
    }
}