﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.PresentationWPF.SupervisorManagement
{
    using System.ComponentModel.Composition;
    using System.Windows;

    using Caliburn.Micro;
    using TFC.PublicLighting.PresentationWPF.Events;
    using TFC.PublicLighting.PresentationWPF.Supervisor;

    [Export(typeof(SupervisorManagementViewModel))]
    public class SupervisorManagementViewModel : Conductor<object>, IHandle<CommandEvent>
    {
        private readonly IWindowManager windowManager;
        private IEventAggregator events;

        [ImportingConstructor]
        public SupervisorManagementViewModel(IEventAggregator pEvents, IWindowManager pWindowManager)
        {
            // Get the event aggregator through the constructor and subscribe this ViewModel so it can listen for Event messages.
            events = pEvents;
            windowManager = pWindowManager;
            events.Subscribe(this);

            bool isNewSupervisor = false;
            ActivateItem(new SupervisorViewModel(events, windowManager, isNewSupervisor));
        }

        # region Commands
        public void ShowSupervisor()
        {
            bool isNewSupervisor = false;
            ActivateItem(new SupervisorViewModel(events, windowManager, isNewSupervisor));
        }
        
        public void ShowNewSupervisor()
        {
            string message = "Estàs segur que vols esborrar el Supervisor actual?";
            string caption = "Confirmació";
            MessageBoxButton buttons = MessageBoxButton.YesNo;
            MessageBoxImage icon = MessageBoxImage.Question;
            if (MessageBox.Show(message, caption, buttons, icon) == MessageBoxResult.Yes)
            {
                // OK codi
                bool isNewSupervisor = true;
                ActivateItem(new SupervisorViewModel(events, windowManager, isNewSupervisor));
            }
        }
        # endregion

        # region Events
        public void Handle(CommandEvent pCommandEvent)
        {
            if (pCommandEvent.CommandToExecute.Equals("ShowNewSupervisor"))
            {
                ShowNewSupervisor();
            }
            else if (pCommandEvent.CommandToExecute.Equals("ShowSupervisor"))
            {
                ShowSupervisor();
            }
        }
        # endregion
    }
}
