﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.PresentationWPF.Tecnic
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.ComponentModel.Composition;
    using System.Text.RegularExpressions;
    using System.Windows;
    using Caliburn.Micro;
    using TFC.PublicLighting.PresentationWPF.Cache;
    using TFC.PublicLighting.PresentationWPF.Events;
    using TFC.PublicLighting.PresentationWPF.PLManagementServiceReference;
    
    [Export(typeof(TecnicViewModel))]
    public class TecnicViewModel : Screen, IDataErrorInfo, IHandle<CommandEvent>
    {
        private readonly IWindowManager windowManager;
        private IEventAggregator events;

        private string _formErrors;
        private string _lastError = "Formulari incomplert o amb errors";
        private List<SectorDTO> _sectorsList;
        private string _controlsAdministrador;

        // SERVEI CLIENT per interactuar amb els serveis distribuits.
        PLManagementServiceClient client = new PLManagementServiceClient();

        // MODEL de dades del ViewModel
        private UsuariDTO _tecnicModel;
           
        [ImportingConstructor]
        public TecnicViewModel(IEventAggregator pEvents, IWindowManager pWindowManager)
        {
            // Obté l'agregador d'esdeveniments a través del constructor i subscriu aquest ViewModel
            // perquè pugui escoltar els missatges d'esdeveniments.
            events = pEvents;
            events.Subscribe(this);

            windowManager = pWindowManager;

            // Comproba quin tipus d'usuari és per activar els controls corresponents.
            if (CacheMain.IsAdministradorOn)
            {
                ControlsAdministrador = "Visible";
            }
            else
            {
                ControlsAdministrador = "Collapsed";
            }

            SectorsList = client.GetAllSectors();
            SectorDTO noSector = new SectorDTO() { IdSector = 0, NomBarri = "-" };
            SectorsList.Insert(0, noSector);
        }

        # region VIEW

        // Accions a efectuar quan la vista ha estat enllaçada al ViewModel
        protected override void OnViewAttached(object view, object context)
        {
            base.OnViewAttached(view, context);

            // Comprova si s'ha de fer servir el caché del model.
            if (CacheMain.IsNewTecnic)
            {
                TecnicModel = new UsuariDTO() { Id = 0, 
                                                TipusDocId = "DNI", 
                                                DataNeixement = DateTime.Now 
                                              };
                CacheMain.IsNewTecnic = false;
                CacheMain.CurrentTecnic = TecnicModel;
            }
            else
            {
                TecnicModel = CacheMain.CurrentTecnic;
            }
        }

        // Accions a efectuar quan es surt de la vista
        protected override void OnDeactivate(bool close)
        {
            CacheMain.CurrentTecnic = TecnicModel;
            base.OnDeactivate(close);
        }

        # endregion

        # region Error
        public string FormErrors
        {
            get { return _formErrors; }
            set
            {
                _formErrors = value;
                NotifyOfPropertyChange(() => FormErrors);
            }
        }

        public string LastError
        {
            get { return _lastError; }
            set
            {
                _lastError = value;
                NotifyOfPropertyChange(() => LastError);
            }
        }
        # endregion

        # region Properties

        public UsuariDTO TecnicModel
        {
            get { return _tecnicModel; }
            set
            {
                _tecnicModel = value;
                NotifyOfPropertyChange(() => TecnicModel);
            }
        }

        public string ControlsAdministrador
        {
            get { return _controlsAdministrador; }
            set
            {
                _controlsAdministrador = value;
                NotifyOfPropertyChange(() => ControlsAdministrador);
            }
        }
                
        /// <summary>
        /// Identificació del Tècnic a la BBDD
        /// </summary>
        public int Id
        {
            get { return TecnicModel.Id; }
            set
            {
                TecnicModel.Id = value;
                NotifyOfPropertyChange(() => Id);
                
                if (value < 0)
                {
                    LastError = "L'Id és incorrecte";
                    _dataErrors["Id"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("Id"))
                        _dataErrors.Remove("Id");
                }
            NotifyOfPropertyChange(() => CanSave);
            }
        }
     
        public string Nom
        {
            get { return TecnicModel.Nom; }
            set
            {
                TecnicModel.Nom = value;
                NotifyOfPropertyChange(() => Nom);
                NotifyOfPropertyChange(() => CanSave);  

                if (string.IsNullOrEmpty(value))
                {
                    LastError = "Es requereix el Nom";
                    _dataErrors["Nom"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("Nom"))
                        _dataErrors.Remove("Nom");
                } 
            }
        }

        public string Cognom1
        {
            get { return TecnicModel.Cognom1; }
            set
            {
                TecnicModel.Cognom1 = value;
                NotifyOfPropertyChange(() => Cognom1);
                NotifyOfPropertyChange(() => CanSave);
                
                if (string.IsNullOrEmpty(value))
                {
                    LastError = "Es requereix el Primer Cognom";
                    _dataErrors["Cognom1"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("Cognom1"))
                        _dataErrors.Remove("Cognom1");
                }
            }
        }

        public string Cognom2
        {
            get { return TecnicModel.Cognom2; }
            set
            {
                TecnicModel.Cognom2 = value;
                NotifyOfPropertyChange(() => Cognom2);
            }
        }
        
        public List<string> TipusDocIdList
        {
            get
            {
                return new List<string> { "DNI", "NIE" };
            }
        }

        public string TipusDocId
        {
            get { return TecnicModel.TipusDocId; }
            set
            {
                TecnicModel.TipusDocId = value;
                NotifyOfPropertyChange(() => TipusDocId);
                NotifyOfPropertyChange(() => CanSave);
                
                if (string.IsNullOrEmpty(value))
                {
                    LastError = "Es requereix el tipus de Document d'Identitat";
                    _dataErrors["TipusDocId"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("TipusDocId"))
                        _dataErrors.Remove("TipusDocId");

                    if (!ValidateDocumentId(TipusDocId, DocumentId))
                    {
                        LastError = "Es requereix un " + TipusDocId + " vàlid";
                        _dataErrors["DocumentId"] = LastError;
                    }
                    else
                    {
                        if (_dataErrors.ContainsKey("DocumentId"))
                            _dataErrors.Remove("DocumentId");
                    }
                }
            }
        }

        public string DocumentId
        {
            get { return TecnicModel.DocumentId; }
            set
            {
                TecnicModel.DocumentId = value;
                NotifyOfPropertyChange(() => DocumentId);
                NotifyOfPropertyChange(() => CanSave);
                
                if (!ValidateDocumentId(TipusDocId, DocumentId))
                {
                    LastError = "Es requereix un " + TipusDocId + " vàlid";
                    _dataErrors["DocumentId"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("DocumentId"))
                        _dataErrors.Remove("DocumentId");
                }
            }
        }

        public DateTime? DataNeixement
        {
            get { return TecnicModel.DataNeixement; }
            set
            {
                TecnicModel.DataNeixement = value;
                NotifyOfPropertyChange(() => DataNeixement);
            }
        }

        public string Categoria
        {
            get { return TecnicModel.Categoria; }
            set
            {
                TecnicModel.Categoria = value;
                NotifyOfPropertyChange(() => Categoria);
            }
        }

        public string Adreca
        {
            get { return TecnicModel.Adreca; }
            set
            {
                TecnicModel.Adreca = value;
                NotifyOfPropertyChange(() => Adreca);
            }
        }

        public string Poblacio
        {
            get { return TecnicModel.Poblacio; }
            set
            {
                TecnicModel.Poblacio = value;
                NotifyOfPropertyChange(() => Poblacio);
            }
        }

        public string CodiPostal
        {
            get { return TecnicModel.CodiPostal; }
            set
            {
                TecnicModel.CodiPostal = value;
                NotifyOfPropertyChange(() => CodiPostal);
                NotifyOfPropertyChange(() => CanSave);

                if (!ValidateCodiPostal(CodiPostal))
                {
                    LastError = "Codi Postal invalid";
                    _dataErrors["CodiPostal"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("CodiPostal"))
                        _dataErrors.Remove("CodiPostal");
                }
            }
        }

        public string Provincia
        {
            get { return TecnicModel.Provincia; }
            set
            {
                TecnicModel.Provincia = value;
                NotifyOfPropertyChange(() => Provincia);
            }
        }

        public string TelefonPrivat
        {
            get { return TecnicModel.TelefonPrivat; }
            set
            {
                TecnicModel.TelefonPrivat = value;
                NotifyOfPropertyChange(() => TelefonPrivat);
                NotifyOfPropertyChange(() => CanSave);

                if (!ValidateTelefon(value))
                {
                    LastError = "Telèfon invalid";
                    _dataErrors["TelefonPrivat"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("TelefonPrivat"))
                        _dataErrors.Remove("TelefonPrivat");
                }
            }
        }

        public string MobilEmpresa
        {
            get { return TecnicModel.MobilEmpresa; }
            set
            {
                TecnicModel.MobilEmpresa = value;
                NotifyOfPropertyChange(() => MobilEmpresa);
                NotifyOfPropertyChange(() => CanSave);

                if (!ValidateMobilEmpresa(value))
                {
                    LastError = "Es requereix un Telef. mobil d'empresa vàlid";
                    _dataErrors["MobilEmpresa"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("MobilEmpresa"))
                        _dataErrors.Remove("MobilEmpresa");
                }
            }
        }

        public string Email
        {
            get { return TecnicModel.Email; }
            set
            {
                TecnicModel.Email = value;
                NotifyOfPropertyChange(() => Email);
                NotifyOfPropertyChange(() => CanSave);

                if (!ValidateEmail(Email))
                {
                    LastError = "Email incorrecte";
                    _dataErrors["Email"] = LastError;
                }                
                else
                {
                    if (_dataErrors.ContainsKey("Email"))
                        _dataErrors.Remove("Email");
                }
            }
        }

        public string Usuari
        {
            get { return TecnicModel.Usuari; }
            set
            {
                TecnicModel.Usuari = value;
                NotifyOfPropertyChange(() => Usuari);
                NotifyOfPropertyChange(() => CanSave);

                if (!ValidateIdentificacio(Usuari))
                {
                    LastError = "Es requereix el nom d'Usuari entre 4 i 12 caràcters";
                    _dataErrors["Usuari"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("Usuari"))
                        _dataErrors.Remove("Usuari");
                }
            }
        }

        public string Contrasenya
        {
            get { return TecnicModel.Contrasenya; }
            set
            {
                TecnicModel.Contrasenya = value;
                NotifyOfPropertyChange(() => Contrasenya);
                NotifyOfPropertyChange(() => CanSave);

                if (!ValidateIdentificacio(Contrasenya))
                {
                    LastError = "Es requereix una Contrasenya entre 4 i 12 caràcters";
                    _dataErrors["Contrasenya"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("Contrasenya"))
                        _dataErrors.Remove("Contrasenya");
                }
            }
        }

        public List<SectorDTO> SectorsList
        {
            get
            {
                return _sectorsList;
            }
            set
            {
                _sectorsList = value;
                NotifyOfPropertyChange(() => SectorsList);
            }
        }
        
        public int Sector1
        {
            get { return TecnicModel.Sector1; }
            set
            {
                int intValue = value;
                TecnicModel.Sector1 = intValue;
                NotifyOfPropertyChange(() => Sector1);
                
                if (intValue == 0)
                {
                    LastError = "El Sector 1 es obligatori";
                    _dataErrors["Sector"] = LastError;
                }
                else if ( (intValue == Sector2) || (intValue == Sector3) || ((Sector2 == Sector3) && (Sector2 != 0)) )
                {
                    LastError = "Sectors repetits";
                    _dataErrors["Sector"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("Sector"))
                        _dataErrors.Remove("Sector");
                }

                NotifyOfPropertyChange(() => CanSave);
            }
        }

        public int Sector2
        {
            get { return TecnicModel.Sector2; }
            set
            {
                int intValue = value;
                TecnicModel.Sector2 = intValue;
                NotifyOfPropertyChange(() => Sector2);

                if ((intValue != 0) && ( (intValue == Sector1) || (intValue == Sector3) || ((Sector1 == Sector3) && (Sector1 != 0)) ))
                {
                    LastError = "Sectors repetits";
                    _dataErrors["Sector"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("Sector"))
                        _dataErrors.Remove("Sector");
                }

                NotifyOfPropertyChange(() => CanSave);
            }
        }

        public int Sector3
        {
            get { return TecnicModel.Sector3; }
            set
            {
                int intValue = value;
                TecnicModel.Sector3 = intValue;
                NotifyOfPropertyChange(() => Sector3);


                if ((intValue != 0) && ((intValue == Sector1) || (intValue == Sector2) || ((Sector1 == Sector2) && (Sector1 != 0))))
                {
                    LastError = "Sectors repetits";
                    _dataErrors["Sector"] = LastError;
                }
                else
                {
                    if (_dataErrors.ContainsKey("Sector"))
                        _dataErrors.Remove("Sector");
                }

                NotifyOfPropertyChange(() => CanSave);
            }
        }

        # endregion

        # region Commands

        public void UpdateById()
        {
            if (Id > 0)
                {
                    TecnicModel = client.GetTecnicById(Id);
                    NotifyOfPropertyChange(string.Empty);
                }
        }

        public void New()
        {
            events.Publish(new CommandEvent { CommandToExecute = "ShowNewTecnic" });
        }
                                               
        public bool CanSave
        {
            get 
            {
                bool isValid = !String.IsNullOrEmpty(Nom) && !String.IsNullOrEmpty(Cognom1) && !String.IsNullOrEmpty(TipusDocId)
                                && ValidateDocumentId(TipusDocId, DocumentId) && Sector1 > 0 
                                && ValidateIdentificacio(Usuari) && ValidateIdentificacio(Contrasenya) && ValidateEmail(Email)
                                && ValidateMobilEmpresa(MobilEmpresa) && ValidateTelefon(TelefonPrivat) && ValidateCodiPostal(CodiPostal);

                return isValid;
            }
        }

        public void Save()
        {
            if (TecnicModel.Id == 0)
            {
                if (client.AddTecnic(TecnicModel, CacheMain.CurrentUserId))
                {
                    MessageBox.Show("Tècnic enregistrat correctament");
                    New();
                }
                else
                {
                    MessageBox.Show("Error enregistrant Tècnic");
                }
            }
            else if (TecnicModel.Id > 0)
            {
                if (client.UpdateTecnic(TecnicModel, CacheMain.CurrentUserId))
                {
                    MessageBox.Show("Tècnic actualitzat correctament");
                    New();
                }
                else
                {
                    MessageBox.Show("Error actualitzant Tècnic");
                }
            }
            NotifyOfPropertyChange(LastError);
            NotifyOfPropertyChange(FormErrors);
        }

        public void Delete()
        {
            if (Id > 0)
            {
                if (client.DeleteTecnic(Id, CacheMain.CurrentUserId))
                {
                    MessageBox.Show("Tècnic esborrat correctament");
                    // Si l'esborrat ha sigut correcte s'obre un nou formulari.
                    New();
                }
                else
                {
                    MessageBox.Show("Error esborrant Tècnic");
                }
            }
        }

        public void Cancel()
        {
            New();
        }

        # endregion

        # region Events
        public void Handle(CommandEvent pCommandEvent)
        {
            if (pCommandEvent.CommandToExecute.Equals("DeleteTecnic"))
            {
                Delete();
            }
        }
        # endregion

        #region IDataErrorInfo Members

        private string _dataError = string.Empty;

        public string Error
        {
            get { return _dataError; }
        }

        protected Dictionary<string, string> _dataErrors =
                       new Dictionary<string, string>();

        public string this[string columnName]
        {
            get
            {
                //Activa o desactiva el missatge general d'errors
                if (_dataErrors.Count != 0)
                {
                    FormErrors = "Visible";
                    NotifyOfPropertyChange(() => FormErrors);
                }
                else
                {
                    FormErrors = "Collapsed";
                    NotifyOfPropertyChange(() => FormErrors);
                }
                
                if (_dataErrors.ContainsKey(columnName))
                {
                    return _dataErrors[columnName];
                }

                else
                    return null;
            }
        }

        #endregion

        # region Auxiliar Methods

        private bool ValidateEmail(string email)
        {
            Regex emailRegex = new Regex(@"^([\w\.\-]+)@([\w\-]+)((\.(\w){2,3})+)$");
            bool isValid = string.IsNullOrEmpty(email) || emailRegex.Match(email).Success;           
            return isValid;
        }

        private bool ValidateDni(string dni)
        {
            if (dni != null)
            {
                Regex dniRegex = new Regex(@"^[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][A-Z]$");
                bool isValid = dniRegex.Match(dni).Success;
                return isValid;
            }
            else return false;
        }

        private bool ValidateNie(string nie)
        {
            Regex nieRegex = new Regex(@"^[XYZ][0-9][0-9][0-9][0-9][0-9][0-9][0-9][A-Z]$");
            bool isValid = nieRegex.Match(nie).Success;
            return isValid;
        }

        private bool ValidateDocumentId(string tipus, string document)
        {
            if (tipus == "DNI")
            {
                return ValidateDni(document);
            }
            else if (tipus == "NIE")
            {
                return ValidateNie(document);
            }
            else return false;
        }

        private bool ValidateIdentificacio(string identificacio)
        {
            return !(string.IsNullOrEmpty(identificacio) || identificacio.Length < 4 || identificacio.Length > 12);
        }

        private bool ValidateCodiPostal(string codiPostal)
        {
            Regex nieRegex = new Regex(@"^[0-9][0-9][0-9][0-9][0-9]$");
            bool isValid = string.IsNullOrEmpty(codiPostal) || nieRegex.Match(codiPostal).Success;
            return isValid;
        }

        private bool ValidateTelefon(string codiPostal)
        {
            Regex telefRegex = new Regex(@"^[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]$");
            bool isValid = string.IsNullOrEmpty(codiPostal) || telefRegex.Match(codiPostal).Success;
            return isValid;
        }

        private bool ValidateMobilEmpresa(string codiPostal)
        {
            Regex telefRegex = new Regex(@"^[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]$");
            bool isValid = telefRegex.Match(codiPostal).Success;
            return isValid;
        }

        # endregion
    }
}