﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.PresentationWPF.TecnicSearch
{
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.ComponentModel.Composition;
    using Caliburn.Micro;
    using TFC.PublicLighting.PresentationWPF.Cache;
    using TFC.PublicLighting.PresentationWPF.Events;
    using TFC.PublicLighting.PresentationWPF.PLManagementServiceReference;

    [Export(typeof(TecnicSearchViewModel))]
    public class TecnicSearchViewModel : Screen
    {
        private readonly IWindowManager windowManager;
        private IEventAggregator events;

        private List<SectorDTO> _sectorsList;
        private List<UsuariDTO> _tecnicsList;
        private UsuariDTO _selectedTecnic;
        
        // SERVEI CLIENT per interactuar amb els serveis distribuits.
        PLManagementServiceClient client = new PLManagementServiceClient();

        // MODEL de dades del ViewModel
        private UsuariDTO _searchTecnicModel = new UsuariDTO();

        [ImportingConstructor]
        public TecnicSearchViewModel(IEventAggregator pEvents, IWindowManager pWindowManager)
        {
            // Obté l'agregador d'esdeveniments a través del constructor i subscriu aquest ViewModel
            // perquè pugui escoltar els missatges d'esdeveniments.
            events = pEvents;
            events.Subscribe(this);

            windowManager = pWindowManager;

            // Obté la llista dels sectors de la xarxa per que estigui disponible a la vista.
            SectorsList = client.GetAllSectors();
            SectorDTO noSector = new SectorDTO() { IdSector = 0, NomBarri = "-" };
            SectorsList.Insert(0, noSector);

            // Obté la llista de tots els tècnics.
            TecnicsList = client.GetAllTecnics();
        }

        # region Properties

        public UsuariDTO TecnicSearchModel
        {
            get { return _searchTecnicModel; }
            set
            {
                _searchTecnicModel = value;
                NotifyOfPropertyChange(() => TecnicSearchModel);
                NotifyOfPropertyChange(string.Empty);
            }
        }

        /// <summary>
        /// Tècnic Seleccionat
        /// </summary>
        public UsuariDTO SelectedTecnic
        {
            get { return _selectedTecnic; }
            set
            {
                CacheMain.CurrentTecnic = value;
                CacheMain.IsNewTecnic = false; 
                _selectedTecnic = value;
                NotifyOfPropertyChange(() => SelectedTecnic);                                               
            }
        }

        public int Id
        {
            get { return TecnicSearchModel.Id; }
            set
            {
                TecnicSearchModel.Id = value;
                NotifyOfPropertyChange(() => Id);
            }
        }

        public string Nom
        {
            get { return TecnicSearchModel.Nom; }
            set
            {
                TecnicSearchModel.Nom = value;
                NotifyOfPropertyChange(() => Nom);
            }
        }

        public string Cognom1
        {
            get { return TecnicSearchModel.Cognom1; }
            set
            {
                TecnicSearchModel.Cognom1 = value;
                NotifyOfPropertyChange(() => Cognom1);
            }
        }

        public string DocumentId
        {
            get { return TecnicSearchModel.DocumentId; }
            set
            {
                TecnicSearchModel.DocumentId = value;
                NotifyOfPropertyChange(() => DocumentId);
            }
        }

        public List<SectorDTO> SectorsList
        {
            get
            {
                return _sectorsList;
            }
            set
            {
                _sectorsList = value;
                NotifyOfPropertyChange(() => SectorsList);
            }
        }

        public List<UsuariDTO> TecnicsList
        {
            get
            {
                return _tecnicsList;
            }
            set
            {
                _tecnicsList = value;
                NotifyOfPropertyChange(() => TecnicsList);
            }
        }

        public int Sector
        {
            get { return TecnicSearchModel.Sector1; }
            set
            {
                int intValue = value;
                TecnicSearchModel.Sector1 = intValue;
                NotifyOfPropertyChange(() => Sector);
            }
        }

        # endregion

        # region Commands

        public void Update()
        {
            TecnicsList = client.GetTecnicsByFilter(TecnicSearchModel);
        }

        public void SelectTecnic()
        {
            CacheMain.IsNewTecnic = false;
            events.Publish(new CommandEvent { CommandToExecute = "ShowTecnic" });
        }

        # endregion
    }
}
