﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.PresentationWPF.TecnicsManagement
{
    using System.ComponentModel.Composition;
    using System.Windows;

    using Caliburn.Micro;
    using TFC.PublicLighting.PresentationWPF.Cache;
    using TFC.PublicLighting.PresentationWPF.Events;
    using TFC.PublicLighting.PresentationWPF.Supervisor;
    using TFC.PublicLighting.PresentationWPF.Tecnic;
    using TFC.PublicLighting.PresentationWPF.TecnicSearch;

    [Export(typeof(TecnicsManagementViewModel))]
    public class TecnicsManagementViewModel : Conductor<object>, IHandle<CommandEvent>
    {
        private readonly IWindowManager windowManager;
        private IEventAggregator events;

        private string _controlsAdministrador;

        [ImportingConstructor]
        public TecnicsManagementViewModel(IEventAggregator pEvents, IWindowManager pWindowManager)
        {
            // Obté l'agregador d'esdeveniments a través del constructor i subscriu aquest ViewModel
            // perquè pugui escoltar els missatges d'esdeveniments.
            events = pEvents;
            events.Subscribe(this);

            windowManager = pWindowManager;

            // Comproba quin tipus d'usuari és per activar els controls corresponents.
            if (CacheMain.IsAdministradorOn)
            {
                ControlsAdministrador = "Visible";
            }
            else
            {
                ControlsAdministrador = "Collapsed";
            }

            ShowTecnic();
        }

        public string ControlsAdministrador
        {
            get { return _controlsAdministrador; }
            set
            {
                _controlsAdministrador = value;
                NotifyOfPropertyChange(() => ControlsAdministrador);
            }
        }

        # region Commands
        public void ShowSearchTecnic()
        {
            ActivateItem(new TecnicSearchViewModel(events, windowManager));
        }

        public void DeleteTecnic()
        {
            string message = "Estàs segur que vols esborrar el Tècnic actual?";
            string caption = "Confirmació";
            MessageBoxButton buttons = MessageBoxButton.YesNo;
            MessageBoxImage icon = MessageBoxImage.Question;
            if (MessageBox.Show(message, caption, buttons, icon) == MessageBoxResult.Yes)
            {
                // OK codi
                events.Publish(new CommandEvent { CommandToExecute = "DeleteTecnic" });
            }            
        }

        public void ShowTecnic()
        {
            ActivateItem(new TecnicViewModel(events, windowManager));
        }

        public void ShowNewTecnic()
        {
            CacheMain.IsNewTecnic = true;
            ActivateItem(new TecnicViewModel(events, windowManager));
        }
        # endregion

        # region Events
        public void Handle(CommandEvent pCommandEvent)
        {
            if (pCommandEvent.CommandToExecute.Equals("ShowNewTecnic"))
            {
                ShowNewTecnic();
            }
            else if (pCommandEvent.CommandToExecute.Equals("ShowTecnic"))
            {
                ShowTecnic();
            }
        }
        # endregion
    }
}
