﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.PresentationWPH7.MainServices
{
    using System.Collections.Generic;
    using System.IO;
    using System.IO.IsolatedStorage;
    using System.Runtime.Serialization;

    public static class LocalStorage
    {
        // Enregistra al fitxer indicat una llista d'objectes que permet tenir altres llistes niades.
        public static void Store<T>(List<T> data, string fileName) where T : class, new()
        {
           using(var storage = IsolatedStorageFile.GetUserStoreForApplication())
           {
              using(var file = storage.OpenFile(fileName, FileMode.OpenOrCreate))
              {
                 var serializer = new DataContractSerializer(typeof(List<T>));
                 serializer.WriteObject(file, data);
              }
           }
        }

        // Llegeix una llista d'objectes del fitxer indicat.
        public static List<T> Load<T>(string fileName) where T : class, new()
        {
           List<T> data = null;
           using(var storage = IsolatedStorageFile.GetUserStoreForApplication())
           {               
               using(var file = storage.OpenFile(fileName, FileMode.OpenOrCreate)) 
              {
                 // If the data was never previously serialized, create new instance.
                 if (file.Length == 0)
                 {
                    data = new List<T>();
                 }

                 var serializer = new DataContractSerializer(typeof(List<T>));
                 data = serializer.ReadObject(file) as List<T>;
              }
           }

           return data;
        }

        // Comprova si el fitxer indicat existeix
        public static bool FileExists (string fileName)
        {
            using (var storage = IsolatedStorageFile.GetUserStoreForApplication())
            {
                bool response = storage.FileExists(fileName);
                return response;
            }
        }

        // Esborra el fitxer indicat existeix
        public static void DeleteFile(string fileName)
        {
            using (var storage = IsolatedStorageFile.GetUserStoreForApplication())
            {
                if (storage.FileExists(fileName))
                {
                    storage.DeleteFile(fileName);
                }
            }
        }
    }
} 


