﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.PresentationWPH7
{
    using System;
    using System.Windows;
    using Microsoft.Phone.Controls;
    using System.Collections.Generic;
    using TFC.PublicLighting.PresentationWPH7.Cache;
    using TFC.PublicLighting.PresentationWPH7.MainServices;
    using TFC.PublicLighting.PresentationWPH7.WPhoneServiceReference;

    public partial class Page_OrdresMain : PhoneApplicationPage
    {
        public Page_OrdresMain()
        {
            InitializeComponent();
            CacheMain.ClearCache();
        }

        private void ListOrdresBT_Click(object sender, RoutedEventArgs e)
        {            
            // SERVEI CLIENT per interactuar amb els serveis distribuits.
            PLManagementServiceClient client = new PLManagementServiceClient();

            MessageBox.Show("Sobrescriu les dades locals."); 

            client.GetAllOrdresByTecnicIdAsync(CacheMain.CurrentUserId);
            client.GetAllOrdresByTecnicIdCompleted += new EventHandler<GetAllOrdresByTecnicIdCompletedEventArgs>(client_GetAllOrdresByTecnicIdCompleted);            
        }

        private void SyncOrdresBT_Click(object sender, RoutedEventArgs e)
        {
            MessageBox.Show("Sobrescriu les dades al Servidor.");            
            // En cas que es tinguin dades locals i es tingui connexió, s'enregistren al sistema remot.
            UpLoadOrdres();
        }

        private void LocalOrdresBT_Click(object sender, RoutedEventArgs e)
        {
            MessageBox.Show("Es treballarà amb dades enregistrades locals.");            
            // En cas que no es tingui connexió es treballa amb les dades locals.
            if (LocalStorage.FileExists(CacheMain.FileStorageName))
            {
                // Es llegeixen les dades del fitxer local.
                List<OrdreDTO> localOrdresList = LocalStorage.Load<OrdreDTO>(CacheMain.FileStorageName);

                // Enregistra al sistema la llista d'Ordres recuperada del fitxer.
                if (localOrdresList.Count > 0)
                {
                    // Enregistrament al caché volàtil per fer servir durant la sessió.
                    CacheMain.OrdresList = localOrdresList;

                    // Navegació a la pàgina de llistat d'Ordres.
                    NavigationService.Navigate(new Uri("/Page_OrdresList.xaml", UriKind.Relative));
                }
                else
                {
                    MessageBox.Show("Fitxer buit. Sistema inactiu.");
                }
            }
            else
            {
                MessageBox.Show("No hi han dades locals. Sistema inactiu."); 
            }            
        }

        
        // Gestió del resultat del llistat d'Ordres
        private void client_GetAllOrdresByTecnicIdCompleted(object sender, GetAllOrdresByTecnicIdCompletedEventArgs e)
        {
            List<OrdreDTO> ordresList = e.Result;

            if (ordresList != null)
            {
                // Enregistrament al caché volàtil per fer servir durant la sessió.
                CacheMain.OrdresList = ordresList;

                if (LocalStorage.FileExists(CacheMain.FileStorageName))
                {
                    // Esborra el fitxer a la memòria permanent isolatedStorage.
                    LocalStorage.DeleteFile(CacheMain.FileStorageName);
                }
               
                // Enregistrament al fitxer a la memòria permanent isolatedStorage.
                LocalStorage.Store(ordresList, CacheMain.FileStorageName);
                               
                // Navegació a la pàgina de llistat d'Ordres.
                NavigationService.Navigate(new Uri("/Page_OrdresList.xaml", UriKind.Relative));
            }
            else
            {
                MessageBox.Show("No es disposen d'Ordres");
            }
        }

        // Actualitza un conjunt d'Ordres al Servidor des del fitxer local.
        private void UpLoadOrdres()
        {
            PLManagementServiceClient client = new PLManagementServiceClient();

            if (LocalStorage.FileExists(CacheMain.FileStorageName))
            {
                // Es llegeixen les dades del fitxer local.
                List<OrdreDTO> ordresListToUpload = LocalStorage.Load<OrdreDTO>(CacheMain.FileStorageName);

                //List<OrdreDTO> ordresListToUpload = CacheMain.OrdresList;
                
                // Enregistra al sistema la llista d'Ordres recuperada del fitxer.
                if (ordresListToUpload.Count > 0)
                {
                    // S'envia la llista d'ordres al servei WCF
                    client.UpdateListOrdresAsync(ordresListToUpload);
                    client.UpdateListOrdresCompleted += new EventHandler<UpdateListOrdresCompletedEventArgs>(client_UpdateListOrdresCompleted);
                }                
            }        
        }

        // Gestió del resultat del llistat d'Ordres
        private void client_UpdateListOrdresCompleted(object sender, UpdateListOrdresCompletedEventArgs e)
        {
            MessageBox.Show("Dades actualitzades al sistema remot.");
        }





    }
}