﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.Services
{
    using System;
    using System.Collections.Generic;
    using System.Transactions;
    using TFC.PublicLighting.Contracts.DTOs;
    using TFC.PublicLighting.Domain;
    using TFC.PublicLighting.Infraestructure;
    using TFC.PublicLighting.Services.Mappers;

    public class ElementsManagementService
    {
        ElementsRepository repository = new ElementsRepository();

        public List<TipusElementDTO> GetAllTipusElement()
        {
            List<TipusElementDTO> allTipusDTOList = new List<TipusElementDTO>();

            List<tipusElement> result = repository.GetAllTipusElement();
            foreach (tipusElement e in result)
            {
                allTipusDTOList.Add(new TipusElementDTO() 
                                    { 
                                        Id = e.id_tipusElement,
                                        Nom = e.tipus 
                                    });
            }
            return allTipusDTOList;
        }

        /// <summary>
        /// Obté tots els elements d'il·luminació.
        /// </summary>
        /// <returns></returns>
        public List<ElementDTO> GetAllElements()
        {
            List<ElementDTO> allElementsDTOList = new List<ElementDTO>();

            List<elementEnllumenat> elements = repository.GetAllElements();
            foreach(elementEnllumenat e in elements)
            {
                allElementsDTOList.Add(e.ToDTO());            
            }

            return allElementsDTOList;
        }

        public List<ElementDTO> GetElementsByFilter(ElementDTO elementFilter)
        {
            List<ElementDTO> allElementsDTOList = new List<ElementDTO>();

            List<elementEnllumenat> elements = repository.GetAllElements();
            foreach (elementEnllumenat e in elements)
            {
                // Element no valid si alguna de les condicions del filtre de busca es invàlida.
                bool isNotValid = (elementFilter.Id > 0 && elementFilter.Id != e.id_element)
                                  || (!string.IsNullOrEmpty(elementFilter.Matricula) && elementFilter.Matricula != e.matricula)
                                  || (!string.IsNullOrEmpty(elementFilter.Carrer) && elementFilter.Carrer != e.carrer)
                                  || (elementFilter.Id_sectorXarxa > 0 && elementFilter.Id_sectorXarxa != e.id_sectorXarxa);
                if (!isNotValid)
                {
                    allElementsDTOList.Add(e.ToDTO());
                }
            }
            return allElementsDTOList;
        }

        public ElementDTO GetElementById(int id)
        {
            elementEnllumenat element = repository.GetElement(id);

            ElementDTO elementDTO = element.ToDTO();

            return elementDTO;
        }

        public bool AddElement(ElementDTO element, int usuariDades)
        {
            try
            {
                using (var tx = new TransactionScope(TransactionScopeOption.Required))
                {
                    elementEnllumenat newElement = element.ToDomain();
                    
                    // Valida que l'id es 0, es a dir que es tracta d'un element nou.
                    if (newElement.id_element != 0)
                    {
                        return false;
                    }

                    // Afegeix l'Element.
                    repository.AddElement(newElement, usuariDades);

                    tx.Complete();
                }
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }

        public bool UpdateElement(ElementDTO element, int usuariDades)
        {
            try
            {
                using (var tx = new TransactionScope(TransactionScopeOption.Required))
                {
                    // Comprova que l'element existeix i en cas contrari es rep una excepció.
                    elementEnllumenat currentElement = repository.GetElement(element.Id);

                    // Converteix l'element DTO a element de domini.
                    elementEnllumenat toUpdateElement = element.ToDomain();

                    // Actualitza l'element.
                    repository.UpdateElement(toUpdateElement, usuariDades);

                    tx.Complete();
                }
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }

        public bool DeleteElement(int id, int usuariDades)
        {
            try
            {
                using (var tx = new TransactionScope(TransactionScopeOption.Required))
                {
                    // Esborra l'element.
                    repository.DeleteElement(id, usuariDades);

                    tx.Complete();
                }
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }
    }
}
