﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.Services
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Transactions;
    using TFC.PublicLighting.Contracts.DTOs;
    using TFC.PublicLighting.Domain;
    using TFC.PublicLighting.Infraestructure;
    using TFC.PublicLighting.Services.Mappers;

    public class IncidenciesManagementService
    {
        IncidenciesRepository repository = new IncidenciesRepository();
        private List<TipusAvariaDTO> _avariesList;
        private List<EstatIncidenciaDTO> _estatsList;

        public IncidenciesManagementService()
        {
            _avariesList = GetAllTipusAvaria();
            _estatsList = GetAllEstatsIncidencia();
        }

        public List<TipusAvariaDTO> GetAllTipusAvaria()
        {
            List<TipusAvariaDTO> allTipusList = new List<TipusAvariaDTO>();

            List<tipusAvaria> result = repository.GetAllTipusAvaria();

            result.ForEach(res =>
            {
                allTipusList.Add(res.ToDTO());
            });

            return allTipusList;
        }

        public List<EstatIncidenciaDTO> GetAllEstatsIncidencia()
        {
            List<EstatIncidenciaDTO> allEstatsDTOList = new List<EstatIncidenciaDTO>();

            List<estatIncidencia> estats = repository.GetAllEstatsIncidencia();

            estats.ForEach(res =>
                {
                    allEstatsDTOList.Add(res.ToDTO());
                });

            return allEstatsDTOList;
        }

        public IncidenciaDTO GetIncidencia(int pId)
        {
            incidencia incidencia = repository.GetIncidencia(pId);

            IncidenciaDTO incidenciaDTO = incidencia.ToDTO();

            return incidenciaDTO;
        }

        public List<IncidenciaDTO> GetAllIncidencies()
        {
            List<incidencia> allIncidenciasList = repository.GetAllIncidencies();

            List<IncidenciaDTO> listIncidenciasDTO = new List<IncidenciaDTO>();

            allIncidenciasList.ForEach(inc =>
            {
                listIncidenciasDTO.Add(inc.ToDTO());
            });

            return listIncidenciasDTO;
        }

        public List<IncidenciaDTO> GetIncidenciesByFilter(IncidenciaDTO incidenciaFilter)
        {
            List<IncidenciaDTO> returnList = new List<IncidenciaDTO>();

            List<incidencia> result = repository.GetAllIncidencies();
            foreach (incidencia e in result)
            {
                // Incidència no vàlida si alguna de les condicions del filtre de busca es invàlida.
                bool isNotValid = (incidenciaFilter.Id_incidencia > 0 && incidenciaFilter.Id_incidencia != e.id_incidencia)
                                  || (incidenciaFilter.Id_element > 0 && incidenciaFilter.Id_element != e.id_element);

                if (!isNotValid)
                {
                    returnList.Add(e.ToDTO());
                }
            }
            return returnList;
        }        

        public bool AddIncidencia(IncidenciaDTO incidenciaDTO, int usuariDades)
        {
            try
            {
                using (var tx = new TransactionScope(TransactionScopeOption.Required))
                {
                    incidencia newIncidencia = incidenciaDTO.ToDomain();
                    
                    // Valida que l'id es 0, es a dir que es tracta d'una incidència nova.
                    if (newIncidencia.id_incidencia != 0)
                    {
                        return false;
                    }

                    // Validació de dades
                    if (newIncidencia.id_tipusAvaria == 0 || newIncidencia.id_element == 0)
                    {
                        return false;
                    }

                    // Afegeix l'Incidència.
                    repository.AddIncidencia(newIncidencia, usuariDades);

                    tx.Complete();
                }
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }

        public bool UpdateIncidencia(IncidenciaDTO incidenciaDTO, int usuariDades)
        {
            try
            {
                using (var tx = new TransactionScope(TransactionScopeOption.Required))
                {
                    // Comprova que la incidencia existeix i en cas contrari es rep una excepció.
                    incidencia currentIndidencia = repository.GetIncidencia(incidenciaDTO.Id_incidencia);

                    incidencia toUpdateIncidencia = incidenciaDTO.ToDomain();

                    // Actualitza la incidencia.
                    repository.UpdateIncidencia(toUpdateIncidencia, usuariDades);

                    tx.Complete();
                }
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }

        public bool DeleteIncidencia(int pId, int usuariDades)
        {
            try
            {
                using (var tx = new TransactionScope(TransactionScopeOption.Required))
                {
                    // Comprova que l'incidència existeix i en cas contrari es rep una excepció.
                    incidencia deletedIncidencia = repository.GetIncidencia(pId);

                    // Esborra la incidència.
                    repository.DeleteIncidencia(pId, usuariDades);

                    tx.Complete();
                }
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }

        public TipusAvariaDTO getAvariaById(int id)
        {
            TipusAvariaDTO result = (from e in _avariesList
                                     where e.Id == id
                                     select e).SingleOrDefault();
            return result;
        }

        public EstatIncidenciaDTO getEstatById(int id)
        {
            EstatIncidenciaDTO result = (from e in _estatsList
                                     where e.Id == id
                                     select e).SingleOrDefault();
            return result;
        }

        public IncidenciaDTO getIncidenciaDTO(incidencia inc)
        {
            IncidenciaDTO incidenciaDTO = inc.ToDTO();            
            incidenciaDTO.Avaria = getAvariaById(incidenciaDTO.Id_tipusAvaria);
            
            return incidenciaDTO;
        }
    }
}
