﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.Services
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Transactions;
    using TFC.PublicLighting.Contracts.DTOs;
    using TFC.PublicLighting.Domain;
    using TFC.PublicLighting.Infraestructure;
    using TFC.PublicLighting.Services.Mappers;

    public class OrdresManagementService
    {
        OrdresRepository ordRepository = new OrdresRepository();
        IncidenciesRepository incRepository = new IncidenciesRepository();
        IncidenciesManagementService incService = new IncidenciesManagementService();

        private List<EstatOrdreDTO> _estatsOrdreList;

        public OrdresManagementService()
        {
            _estatsOrdreList = GetAllEstatsOrdre();
        }

        public List<TipusPrioritatDTO> GetAllTipusPrioritat()
        {
            List<TipusPrioritatDTO> returnList = new List<TipusPrioritatDTO>();

            List<tipusPrioritat> result = ordRepository.GetAllTipusPrioritat();
            foreach (tipusPrioritat e in result)
            {
                returnList.Add(new TipusPrioritatDTO()
                {
                    Id = e.id_tipusPrioritat,
                });
            }
            return returnList;
        }

        public List<EstatOrdreDTO> GetAllEstatsOrdre()
        {
            List<EstatOrdreDTO> returnList = new List<EstatOrdreDTO>();

            List<estatOrdre> result = ordRepository.GetAllEstatsOrdre();
            foreach (estatOrdre e in result)
            {
                returnList.Add(new EstatOrdreDTO()
                {
                    Id = e.id_estat_ordre,
                    Nom = e.estat
                });
            }
            return returnList;
        }

        public OrdreDTO GetOrdre(int id)
        {
            ordreReparacio ordre = ordRepository.GetOrdreById(id);

            OrdreDTO ordreDTO = ordre.ToDTO();
            ordreDTO.Estat = getEstatOrdById(ordreDTO.Id_estatOrdre);

            foreach (incidencia i in ordre.incidencies)
            {
                IncidenciaDTO incidenciaDTO = incService.getIncidenciaDTO(i);
                ordreDTO.IncidenciesList.Add(incidenciaDTO);               
            }
            return ordreDTO;
        }

        public List<OrdreDTO> GetAllOrdres()
        {
            List<ordreReparacio> ordres = ordRepository.GetAllOrdres();
            return OrdresListToOrdresDTOList(ordres);
        }

        public List<OrdreDTO> GetOrdresByFilter(OrdreDTO ordreFilter)
        {
            List<OrdreDTO> result = this.GetAllOrdres();
            List<OrdreDTO> returnList = new List<OrdreDTO>();

            foreach (OrdreDTO e in result)
            {
                // Ordre no vàlida si alguna de les condicions del filtre de busca es invàlida.
                bool isNotValid = (ordreFilter.Id_ordreReparacio > 0 && ordreFilter.Id_ordreReparacio != e.Id_ordreReparacio)
                                  || (ordreFilter.Id_tecnic > 0 && ordreFilter.Id_tecnic != e.Id_tecnic)
                                  || (ordreFilter.Id_estatOrdre > 0 && ordreFilter.Id_estatOrdre != e.Id_estatOrdre);

                if (!isNotValid)
                {
                    returnList.Add(e);
                }
            }
            return returnList;
        }

        //public List<OrdreDTO> GetOrdresByFilter(OrdreDTO ordreFilter)
        //{
        //    List<OrdreDTO> returnList = new List<OrdreDTO>();

        //    List<ordreReparacio> result = ordRepository.GetAllOrdres();
        //    foreach (ordreReparacio e in result)
        //    {
        //        // Ordre no vàlida si alguna de les condicions del filtre de busca es invàlida.
        //        bool isNotValid = (ordreFilter.Id_ordreReparacio > 0 && ordreFilter.Id_ordreReparacio != e.id_ordreReparacio)
        //                          || (ordreFilter.Id_tecnic > 0 && ordreFilter.Id_tecnic != e.id_tecnic)
        //                          || (ordreFilter.Id_estatOrdre > 0 && ordreFilter.Id_estatOrdre != e.id_estatOrdre);

        //        if (!isNotValid)
        //        {
        //            returnList.Add(e.ToDTO());
        //        }
        //    }
        //    return returnList;
        //}

        public List<OrdreDTO> GetAllOrdresByTecnicId(int id)
        {
            List<ordreReparacio> ordres = ordRepository.GetAllOrdresByTecnicId(id);
            return OrdresListToOrdresDTOList(ordres);
        }

        public bool AddOrdre(OrdreDTO ordreDTO, int usuariDades)
        {
            try
            {
                using (var tx = new TransactionScope(TransactionScopeOption.Required))
                {
                    ordreReparacio newOrdre = ordreDTO.ToDomain();

                    // Valida que l'id es 0, es a dir que es tracta d'una ordre nova.
                    if (newOrdre.id_ordreReparacio != 0)
                    {
                        return false;
                    }

                    // Validació de dades
                    if (string.IsNullOrEmpty(newOrdre.id_tipusPrioritat) || newOrdre.id_estatOrdre == 0 || newOrdre.id_tecnic == 0)
                    {
                        return false;
                    }

                    // A partir d'aquest punt, no pot venir una llista d'incidències buida.
                    // Crea la llista d'id de les incidències de l'OrdreDTO.
                    List<int> IdIncidenciesList = new List<int>();
                    foreach (IncidenciaDTO i in ordreDTO.IncidenciesList)
                    {
                        IdIncidenciesList.Add(i.Id_incidencia);
                    }

                    // Afegeix l'Ordre
                    ordRepository.AddOrdre(newOrdre, IdIncidenciesList, usuariDades);

                    tx.Complete();
                }
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }

        public bool UpdateOrdre(OrdreDTO ordreToUpdateDTO, int usuariDades)
        {
            try
            {
                using (var tx = new TransactionScope(TransactionScopeOption.Required))
                {                   
                    // Comprova que existeix l'ordre i en cas contrari llença una excepció.
                    ordreReparacio currentOrdre = ordRepository.GetOrdreById(ordreToUpdateDTO.Id_ordreReparacio);

                    // Validació de dades
                    if (string.IsNullOrEmpty(ordreToUpdateDTO.Id_tipusPrioritat) || ordreToUpdateDTO.Id_estatOrdre == 0 || ordreToUpdateDTO.Id_tecnic == 0)
                    {
                        return false;
                    }

                    // Converteix l'ordre DTO a ordre de domini.
                    ordreReparacio ordreToUpdate = ordreToUpdateDTO.ToDomain();

                    // Crea la llista d'id de les incidències de l'OrdreDTO.
                    List<int> IdIncidenciesList = new List<int>();
                    foreach (IncidenciaDTO i in ordreToUpdateDTO.IncidenciesList)
                    {
                        IdIncidenciesList.Add(i.Id_incidencia);
                    }

                    // Actualitza l'ordre.
                    ordRepository.UpdateOrdre(ordreToUpdate, IdIncidenciesList, usuariDades);

                    tx.Complete();
                }
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }

        public bool DeleteOrdre(int pId, int pIdUsuariDades)
        {
            try
            {
                using (var tx = new TransactionScope(TransactionScopeOption.Required))
                {
                    // Comprova que existeix l'ordre i en cas contrari llença una excepció.
                    ordreReparacio currentOrdre = ordRepository.GetOrdreById(pId);

                    // Esborra l'ordre directament.
                    ordRepository.DeleteOrdre(currentOrdre.id_ordreReparacio, pIdUsuariDades);

                    tx.Complete();
                }
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }

        public bool UpdateListOrdres(List<OrdreDTO> ordreDTOList, int usuariDades)
        {
            try
            {
                using (var tx = new TransactionScope(TransactionScopeOption.Required))
                {
                    foreach (OrdreDTO ordreDTO in ordreDTOList)
                    {
                        UpdateOrdre(ordreDTO, usuariDades);
                    }

                    tx.Complete();
                }
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }

        public EstatOrdreDTO getEstatOrdById(int id)
        {
            EstatOrdreDTO result = (from e in _estatsOrdreList
                                         where e.Id == id
                                         select e).SingleOrDefault();
            return result;
        }

        private List<OrdreDTO> OrdresListToOrdresDTOList (List<ordreReparacio> ordresList)
        {
            List<OrdreDTO> ordreDTOList = new List<OrdreDTO>();

            foreach (ordreReparacio ord in ordresList)
            {
                OrdreDTO ordreDTO = ord.ToDTO();
                ordreDTO.Estat = getEstatOrdById(ordreDTO.Id_estatOrdre);

                foreach (incidencia incidencia in ord.incidencies)
                {
                    IncidenciaDTO incidenciaDTO = incService.getIncidenciaDTO(incidencia);
                    ordreDTO.IncidenciesList.Add(incidenciaDTO);
                }
                ordreDTOList.Add(ordreDTO);
            };

            return ordreDTOList;

        }
    }
}
