﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.Services
{
    using System;
    using System.Transactions;
    using TFC.PublicLighting.Contracts.DTOs;
    using TFC.PublicLighting.Domain;
    using TFC.PublicLighting.Infraestructure;
    using TFC.PublicLighting.Services.Mappers;

    public class SupervisorManagementService
    {
        UsuarisRepository usuarisRepository = new UsuarisRepository();

        public UsuariDTO GetSupervisor()
        {
            usuari uSupervisor = usuarisRepository.GetSupervisor();
            UsuariDTO supervisorDTO = DTOMappers.UsuariToUsuariDTO(uSupervisor);
            return supervisorDTO;
        }

        public bool AddSupervisor(UsuariDTO supervisorDTO, int usuariDades)
        {
            using (var tx = new TransactionScope(TransactionScopeOption.Required))
            {
                // Fa un esborrat lógic de tots els usuaris tipus Supervisor
                usuarisRepository.SetUsuarisAsRemovedByType(Constants.UserType.Super, usuariDades);

                // Afegeix el nou Supervisor.
                supervisorDTO.IdTipusUsuari = Constants.UserType.Super;
                usuari uSupervisor = DTOMappers.UsuariDTOToUsuari(supervisorDTO);
                bool result = usuarisRepository.AddUsuari(uSupervisor, usuariDades);

                tx.Complete();
            }
            return true;
        }

        public bool UpdateSupervisor(UsuariDTO supervisorDTO, int usuariDades)
        {
            supervisorDTO.IdTipusUsuari = Constants.UserType.Super;
            usuari uSupervisor = DTOMappers.UsuariDTOToUsuari(supervisorDTO);
            bool succes = usuarisRepository.UpdateUsuari(uSupervisor, usuariDades);
            return succes;
        }

        /// <summary>
        /// Elimina l'usuari Supervisor indicat
        /// </summary>
        /// <param name="pId">L'Id que identifica l'Usuari.</param>
        /// <returns>TRUE / FALSE</returns>
        public bool DeleteSupervisor(int pId, int usuariDades)
        {
            try
            {
                using (var tx = new TransactionScope(TransactionScopeOption.Required))
                {
                    usuari deletedSupervisor = usuarisRepository.GetUserByTypeAndId(pId, Constants.UserType.Super);

                    usuarisRepository.DeleteUsuari(deletedSupervisor.id_usuari, usuariDades);

                    tx.Complete();
                }
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }
    }
}
