﻿// --------------------------------------------------------------------------------------------------------------------
// Sistema Gestió Incidències Enllumenat Públic
// UOC. Treball de Fí de Carrera .NET
//
// Universitat Oberta de Catalunya, Gener 2012
// Toni Navarro Sánchez
// toninavarrosan@gmail.com
// --------------------------------------------------------------------------------------------------------------------

namespace TFC.PublicLighting.Services
{
    using System;
    using System.Collections.Generic;
    using System.Transactions;
    using TFC.PublicLighting.Contracts.DTOs;
    using TFC.PublicLighting.Domain;
    using TFC.PublicLighting.Infraestructure;
    using TFC.PublicLighting.Services.Mappers;

    public class TecnicsManagementService
    {
        UsuarisRepository usuarisRepository = new UsuarisRepository();

        public UsuariDTO GetTecnicById (int pId)
        {
            usuari uTecnic = usuarisRepository.GetTecnicById(pId);
            UsuariDTO tecnicDTO = DTOMappers.UsuariToUsuariDTO(uTecnic);            
            return tecnicDTO;        
        }

        public List<UsuariDTO> GetAllTecnics()
        {
            List<UsuariDTO> returnList = new List<UsuariDTO>();

            List<usuari> result = usuarisRepository.GetAllUsuaris(Constants.UserType.Tec);
            foreach (usuari e in result)
            {
                returnList.Add(DTOMappers.UsuariToUsuariDTO(e));
            }

            return returnList;
        }

        public List<UsuariDTO> GetTecnicsByFilter(UsuariDTO tecnicFilter)
        {
            List<UsuariDTO> returnList = new List<UsuariDTO>();

            List<usuari> result = usuarisRepository.GetAllUsuaris(Constants.UserType.Tec);
            foreach (usuari e in result)
            {
                // Tècnic no vàlid si alguna de les condicions del filtre de busca es invàlida.
                bool isNotValid = (tecnicFilter.Id > 0 && tecnicFilter.Id != e.id_usuari)
                                  || (!string.IsNullOrEmpty(tecnicFilter.Nom) && tecnicFilter.Nom != e.nom)
                                  || (!string.IsNullOrEmpty(tecnicFilter.Cognom1) && tecnicFilter.Cognom1 != e.cognom1)
                                  || (!string.IsNullOrEmpty(tecnicFilter.DocumentId) && tecnicFilter.DocumentId != e.documentId)
                                  || (tecnicFilter.Sector1 > 0 && tecnicFilter.Sector1 != e.id_sector1 && tecnicFilter.Sector1 != e.id_sector2 && tecnicFilter.Sector1 != e.id_sector3);

                if (!isNotValid)
                {
                    returnList.Add(DTOMappers.UsuariToUsuariDTO(e));
                } 
            }
            return returnList;
        }

        public bool AddTecnic(UsuariDTO tecnicDTO, int usuariDades)
        {
            tecnicDTO.IdTipusUsuari = Constants.UserType.Tec;
            usuari uTecnic = DTOMappers.UsuariDTOToUsuari(tecnicDTO);
            bool result = usuarisRepository.AddUsuari(uTecnic, usuariDades);
            return result;
        }

        public bool UpdateTecnic(UsuariDTO tenicDTO, int usuariDades)
        {
            try
            {
                using (var tx = new TransactionScope(TransactionScopeOption.Required))
                {
                    // Comprova que el tècnic existeix i en cas contrari es rep una excepció.
                    usuari currentTecnic = usuarisRepository.GetUserByTypeAndId(tenicDTO.Id, Constants.UserType.Tec);

                    usuari usuari = DTOMappers.UsuariDTOToUsuari(tenicDTO);

                    // Actualitza el tècnic.
                    usuarisRepository.UpdateUsuari(usuari, usuariDades);

                    tx.Complete();
                }
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }

        public bool DeleteTecnic(int pId, int usuariDades)
        {
            try
            {
                using (var tx = new TransactionScope(TransactionScopeOption.Required))
                {
                    // Comprova que el tècnic existeix i en cas contrari es rep una excepció.
                    usuari technicalUser = usuarisRepository.GetUserByTypeAndId(pId, Constants.UserType.Tec);

                    // Esborra el tècnic directament.
                    usuarisRepository.DeleteUsuari(technicalUser.id_usuari, usuariDades);

                    tx.Complete();
                }
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }
    }
}
